from brian import NeuronGroup, SpikeCounter, Network, ms, second, isscalar

"""
This script shows how to distribute a Brian script without effort.

Define a function as follows :

def fun(x, args):
    # ...
    return r

This function defines a NeuronGroup and does some operations before returning
a result r.
'x' is a parameter, and you want to calculate r for different values of x.
Also, there are several static parameters in args that do not change.

If you call
r = fun(x, dict(arg1=val1, ...))

then your function should return the right result. If you want to evaluate fun
with several values of x, then you would do :

xlist = ...
rlist = zeros(len(xlist))
for i in xrange(len(xlist)):
    rlist[i] = fun(xlist[i], dict(arg1=val1, ...)) 
    
This code snippet can be accelerated if you have multiple CPUs by doing this :

from playdoh import *
dfun = distribute(fun, dict(arg1=val1, ...))
xlist = ...
rlist = dfun(xlist)

That's it!
"""

def fun(sigma, args):
    """
    This function computes the mean firing rate of a LIF neuron with
    white noise input current (OU process with threshold).
    """
    if not isscalar(sigma):
        raise Exception('sigma must be a scalar')
    N = args['N']
    tau = args['tau']
    model = args['model']
    reset = args['reset']
    threshold = args['threshold']
    duration = args['duration']
    G = NeuronGroup(N, model=model, reset=reset, threshold=threshold)
    M = SpikeCounter(G)
    net = Network(G, M)
    net.run(duration)
    r = M.nspikes * 1.0 / N
    return r

if __name__ == '__main__':
    from playdoh import *

    N = 2000
    tau = 10 * ms
    model = '''dV/dt=-V/tau+sigma*(2/tau)**.5*xi : 1'''
    reset = 0
    threshold = 1
    duration = 1 * second
    sigmas = [.3, .4, .5, .6]

    dfun = distribute(fun, dict(N=N, tau=tau, duration=duration,
                                       model=model, reset=reset, threshold=threshold))

    rates = dfun(sigmas)
    print rates



from brian.inspection import namespace
import re


class Symbol(str):
    pass


class OutputCode(object):
    def __init__(self, source=None):
        self.code = ''
        if source is None:
            source = self
        self.source = source
        self.curtarget = self

    def newtarget(self):
        self.curtarget = OutputCode(self.curtarget)
        return self.curtarget

    def close(self):
        self.source.code = self.source.code + self.curtarget.code
        self.curtarget = self.curtarget.source

    def add(self, line):
        self.curtarget.code = self.curtarget.code + line

    def __str__(self):
        s = ''
        tabs = 0
        for line in self.code.split('\n'):
            if '}' in line and '{' not in line: tabs -= 4
            s += ' ' * tabs + line + '\n'
            if '{' in line and '}' not in line: tabs += 4
        return s
    __repr__ = __str__


class Maker(object):
    def __init__(self, cls, outputcode):
        self.outputcode = outputcode
        self.cls = cls
        self.instance = 0

    def __iter__(self):
        self.instance += 1
        return self.cls(self.outputcode.newtarget(), self.instance)


class Spikes(object):
    def __init__(self, targetcode, instance=0):
        self.instance = instance
        self.targetcode = targetcode
        self.targetcode.add('for(int spike_index=0; spike_index<spikes_len; spike_index++){\n')
        self.targetcode.add('int neuron_index%d = spikes[spike_index];\n' % instance)
        self.value = 0

    def __iter__(self):
        return self

    def next(self):
        if self.value == 0:
            self.value = 1
            return Symbol('neuron_index' + str(self.instance))
        self.targetcode.add('}\n')
        self.targetcode.close()
        raise StopIteration


class Evaluate(object):
    def __init__(self, outputcode):
        self.outputcode = outputcode

    def __call__(self, code):
        ns = namespace(code, level=1)
        for k, v in ns.iteritems():
            if isinstance(v, Symbol):
                code = re.sub("\\b" + k + "\\b", str(v), code)
        self.outputcode.add(''.join(expr + ';\n' for expr in code.split('\n') if expr.strip()))


class DenseMatrix(object):
    def __init__(self, outputcode, matrix_name):
        self.outputcode = outputcode
        self.matrix_name = matrix_name
        self.instance = 0

    def row(self, row):
        self.instance += 1
        return DenseMatrixRow(row, self.matrix_name,
                              self.outputcode.newtarget(), self.instance)


class DenseMatrixRow(object):
    def __init__(self, row, matrix_name, targetcode, instance):
        self.targetcode = targetcode
        self.instance = instance
        self.row = row
        self.matrix_name = matrix_name
        self.targetcode.add('for(int col_index%d=0; col_index%d<%s_num_cols; col_index%d++){\n' % (instance, instance, matrix_name, instance))
        self.targetcode.add('double &matrix_value%d = %s[col_index%d+%s*%s_num_cols];\n' % (instance, matrix_name, instance, row, matrix_name))
        self.value = 0

    def __iter__(self):
        return self

    def next(self):
        if self.value == 0:
            self.value = 1
            return Symbol('col_index' + str(self.instance)), Symbol('matrix_value' + str(self.instance))
        self.targetcode.add('}\n')
        self.targetcode.close()
        raise StopIteration


class Load(object):
    def __init__(self, outputcode):
        self.outputcode = outputcode

    def __call__(self, code):
        addcode = ''
        for line in code.split('\n'):
            for expr in line.split(';'):
                if expr.strip():
                    m = re.search('(\w*)\[(.*?)\]', expr)
                    if not m:
                        raise ValueError('Load string incorrent format: ' + code)
                    varname = m.group(1)
                    index = m.group(2)
                    addcode += '%s = &%s__array[%s];\n' % (varname, varname, index)
        self.outputcode.add(addcode)

if __name__ == '__main__':
    outputcode = OutputCode()
    spikes = Maker(Spikes, outputcode)
    evaluate = Evaluate(outputcode)
    load = Load(outputcode)
    W = DenseMatrix(outputcode, 'W')

    for i in spikes:
        for j, w in W.row(i):
            load('V[i]')
            evaluate('w += V')

    print outputcode

    # This works as follows, when you do for example:
    #    for i in spikes:
    # the object spikes knows that it should output code to the OutputCode object
    # outputcode. The spikes object has a method __iter__ and this returns a
    # Spikes object s, which follows the iteration protocol so that s.next()
    # returns either the next object in the container or raises StopIteration.
    # In fact, we only use it once, the first time we add some code to outputcode
    # to do the iteration, and return a Symbol object which is a string with
    # the name of the neuron index being iterated over. So for example it adds
    # the code:
    #    for(int spike_index=0; spike_index<spikes_len; spike_index++){
    #        int neuron_index1 = spikes[spike_index];
    # and returns Symbol('neuron_index1') so that i=Symbol('neuron_index1').
    # Now if other code uses 'i' or i then it will be replaced by neuron_index1.
    # The second time the iterator is called it adds the code '}' to outputcode
    # and then raises StopIteration. The effect of this is that the for block
    # is executed precisely once, wrapped inside a C for-loop.
    # The other examples like W.row(i), load('V[i]') and evaluate('w += V') work
    # in a similar way.
    #
    # Questions:
    # * can we generate Python code with the same system?
    # * can we structure the code above to make it simpler? e.g. putting C
    #   and Python code into template files rather than having them mixed
    #   together?



"""
Van Rossum metric.
We express the metric as an STDP rule + counter.

I think for Poisson inputs we get (Fi+Fj)*tau/2 (after normalization by the
duration).
"""
from brian import *

tau = 20 * ms
N = 100
b = 1.2 # constant current mean, the modulation varies
f = 10 * Hz
tau_VR=2*ms

eqs = '''
dv/dt=(-v+a*sin(2*pi*f*t)+b)/tau : 1
a : 1
'''

neurons = NeuronGroup(N, model=eqs, threshold=1, reset=0)
neurons.v = rand(N)
neurons.a = linspace(.05, 0.75, N)
S = SpikeMonitor(neurons)
trace = StateMonitor(neurons, 'v', record=50)
VR=VanRossumMetric(neurons,tau=tau_VR)

# Alternative implementation
class NewVanRossumMetric(STDP):
    def __init__(self,source,tau=2*ms):    
        self.counter=SpikeCounter(neurons)
        self.N=len(neurons)
        class Fake(object):
            def __init__(self,source):
                self.source=source
                self.target=source
                self.delay=0
                self.W=zeros((len(source),len(source)))
        self.synapses=Fake(neurons)
        
        eqs_stdp='''
        dApost/dt=-Apost/tau_VR : 1
        dApre/dt=-Apre/tau_VR : 1
        '''
        STDP.__init__(self,self.synapses,eqs_stdp,pre='Apre+=1;w+=Apost',post='Apost+=1;w+=Apre')
        self.contained_objects.append(self.counter)
        
    def get_distance(self):
        c=self.counter.count*ones((self.N,1))
        return c+c.T-2*self.synapses.W # divide by duration?

VR2=NewVanRossumMetric(neurons,tau=tau_VR)

run(1000 * ms)
subplot(221)
raster_plot(S)
subplot(222)
plot(trace.times / ms, trace[50])
subplot(223)
imshow(VR.get_distance())
subplot(224)
imshow(VR2.get_distance())

show()



'''
Electrophysiology models for Brian.
R. Brette 2008.

Contains:
* Electrode models
* Current clamp and voltage amplifiers
* DCC and SEVC amplifiers (discontinuous)
* AEC (Active Electrode Compensation)
* Acquisition board
'''
from brian.units import amp, check_units, volt, farad
from brian.equations import Equations, unique_id
from operator import isSequenceType
from brian.units import ohm, Mohm
from brian.stdunits import pF, ms, nA, mV, nS
from brian.neurongroup import NeuronGroup
from scipy import zeros, array, optimize, mean, arange, diff, rand, exp, sum, convolve, eye, linalg, sqrt
from brian.clock import Clock
from electrode_compensation import *

__all__ = ['electrode', 'current_clamp', 'voltage_clamp', 'DCC', 'SEVC',
         'AcquisitionBoard', 'AEC', 'VC_AEC']

'''
------------
 Electrodes
------------
'''
# TODO: sharp electrode model (cone)
# No unit checking of Re and Ce because they can be lists
@check_units(v_rec=volt, vm=volt, i_inj=amp, i_cmd=amp)
def electrode(Re, Ce, v_el='v_el', vm='vm', i_inj='i_inj', i_cmd='i_cmd'):
    '''
    An intracellular electrode modeled as an RC circuit,
    or multiple RC circuits in series (if Re, Ce are lists).
    v_el = electrode (=recording) potential
    vm = membrane potential
    i_inj = current entering the membrane
    i_cmd = electrode command current (None = no injection)
    Returns an Equations() object.
    '''
    if isSequenceType(Re):
        if len(Re) != len(Ce) or len(Re) < 2:
            raise TypeError, "Re and Ce must have the same length"
        v_mid, i_mid = [], []
        for i in range(len(Re) - 1):
            v_mid.append('v_mid_' + str(i) + unique_id())
            i_mid.append('i_mid_' + str(i) + unique_id())
        eqs = electrode(Re[0], Ce[0], v_mid[0], vm, i_inj, i_mid[0])
        for i in range(1, len(Re) - 1):
            eqs + electrode(Re[i], Ce[i], v_mid[i], v_mid[i - 1], i_mid[i - 1], i_mid[i])
        eqs += electrode(Re[-1], Ce[-1], v_el, v_mid[-1], i_mid[-1], i_cmd)
        return eqs
    else:
        if Ce > 0 * farad:
            return Equations('''
            dvr/dt = ((vm-vr)/Re+ic)/Ce : mV
            ie = (vr-vm)/Re : nA''', vr=v_el, vm=vm, ic=i_cmd, ie=i_inj, \
            Re=Re, Ce=Ce)
        else: # ideal electrode - pb here
            return Equations('''
            vr = vm+Re*ic : volt
            ie = ic : amp''', vr=v_el, vm=vm, ic=i_cmd, ie=i_inj)

'''
------------
 Amplifiers
------------
'''
def voltage_clamp(vm='vm', v_cmd='v_cmd', i_rec='i_rec',
                  i_inj='i_inj', Re=20 * Mohm, Rs=0 * Mohm, tau_u=1 * ms):
    '''
    Continuous voltage-clamp amplifier + ideal electrode
    (= input capacitance is already neutralized).
    
    vm = membrane potential (or electrode) variable
    v_cmd = command potential
    i_inj = injected current (into the neuron)
    i_rec = recorded current (= -i_inj)
    Re = electrode resistance
    Rs = series resistance compensation
    tau_u = delay of series compensation (for stability)
    '''
    return Equations('''
    Irec=-I : amp
    I=(Vc+U-Vr)/Re : amp
    dU/dt=(Rs*I-U)/tau : volt
    ''', Vr=vm, Vc=v_cmd, I=i_inj, Rs=Rs, tau=tau_u, Irec=i_rec, Re=Re)

# TODO: Re, Ce as lists
def current_clamp(vm='vm', i_inj='i_inj', v_rec='v_rec', i_cmd='i_cmd',
                  Re=80 * Mohm, Ce=4 * pF, bridge=0 * ohm, capa_comp=0 * farad,
                  v_uncomp=None):
    '''
    Continuous current-clamp amplifier + electrode.
    
    vm = membrane potential (or electrode) variable
    i_inj = injected current (into the neuron)
    v_rec = recorded potential
    i_cmd = command current
    bridge = bridge resistance compensation
    capa_comp = capacitance neutralization
    Re = electrode resistance
    Ce = electrode capacitance (input capacitance)
    v_uncomp = uncompensated potential (raw measured potential)
    '''
    if capa_comp != Ce:
        return Equations('''
        Vr=U-R*Ic    : volt
        I=(U-V)/Re : amp
        dU/dt=(Ic-I)/(Ce-CC) : volt
        ''', Vr=v_rec, V=vm, I=i_inj, Ic=i_cmd, R=bridge, Ce=Ce, CC=capa_comp, U=v_uncomp, Re=Re)
    else:
        return Equations('''
        Vr=V+(Re-R)*I    : volt
        I=Ic : amp # not exactly an alias because the units of Ic is unknown
        ''', Vr=v_rec, V=vm, I=i_inj, Ic=i_cmd, R=bridge, Re=Re)


class AcquisitionBoard(NeuronGroup):
    '''
    Digital acquisition board (DSP).
    Use: board=AcquisitionBoard(P=neuron,V='V',I='I',clock)
    where
      P = neuron group
      V = potential variable name (in P)
      I = current variable name (in P)
      clock = acquisition clock
    Recording: vm=board.record
    Injecting: board.command=...
    
    Injects I, records V.
    '''
    def __init__(self, P, V, I, clock=None):
        eqs = Equations('''
        record : units_record
        command : units_command
        ''', units_record=P.unit(V), units_command=P.unit(I))
        NeuronGroup.__init__(self, len(P), model=eqs, clock=clock)
        self._P = P
        self._V = V
        self._I = I

    def update(self):
        self.record = self._P.state(self._V) # Record
        self._P.state(self._I)[:] = self.command # Inject


class DCC(AcquisitionBoard):
    '''
    Discontinuous current-clamp.
    Use: board=DCC(P=neuron,V='V',I='I',frequency=2*kHz)
    where
      P = neuron group
      V = potential variable name (in P)
      I = current variable name (in P)
      frequency = sampling frequency
    Recording: vm=board.record
    Injecting: board.command=I
    '''
    def __init__(self, P, V, I, frequency):
        self.clock = Clock(dt=1. / (3. * frequency))
        AcquisitionBoard.__init__(self, P, V, I, self.clock)
        self._cycle = 0

    def set_frequency(self, frequency):
        '''
        Sets the sampling frequency.
        '''
        self.clock.dt = 1. / (3. * frequency)

    def update(self):
        if self._cycle == 0:
            self.record = self._P.state(self._V) # Record
            self._P.state(self._I)[:] = 3 * self.command # Inject
        else:
            self._P.state(self._I)[:] = 0 #*nA
        self._cycle = (self._cycle + 1) % 3


class SEVC(DCC):
    '''
    Discontinuous voltage-clamp.
    Use: board=SEVC(P=neuron,record='V',command='I',frequency=2*kHz,gain=10*nS)
    where
      P = neuron group
      V = potential variable name (in P)
      I = current variable name (in P)
      frequency = sampling frequency
      gain = feedback gain
      gain2 = control gain (integral controller)
    Recording: i=board.record
    Setting the clamp potential: board.command=-20*mV
    '''
    def __init__(self, P, V, I, frequency, gain=100 * nS, gain2=0 * nS / ms):
        DCC.__init__(self, P, V, I, frequency)
        self._J = zeros(len(P)) # finer control
        self._gain = gain
        self._gain2 = gain2

    def update(self):
        if self._cycle == 0:
            self._J += self.clock._dt * self._gain2 * (self._P.state(self._V) - self.command)
            self.record = self._gain * (self._P.state(self._V) - self.command) + self._J
            self._P.state(self._I)[:] = -3 * self.record # Inject
        else:
            self._P.state(self._I)[:] = 0 #*nA
        self._cycle = (self._cycle + 1) % 3

'''
-------------------------------------
 Active Electrode Compensation (AEC)
 
The technique was presented in the following paper:
High-resolution intracellular recordings using a real-time computational model of the electrode
R. Brette, Z. Piwkowska, C. Monier, M. Rudolph-Lilith, J. Fournier, M. Levy, Y. Fregnac, T. Bal, A. Destexhe
Neuron (2008) 59(3):379-91.

-------------------------------------
'''


class AEC(AcquisitionBoard):
    """
    An acquisition board with AEC.
    Warning: only works with 1 neuron (not N).
    Use: board=AEC(neuron,'V','I',clock)
    where
      P = neuron group
      V = potential variable name (in P)
      I = current variable name (in P)
      clock = acquisition clock
    Recording: vm=board.V
    Injecting: board.command(I)
    """

    def __init__(self, P, V, I, clock=None):
        AcquisitionBoard.__init__(self, P, V, I, clock=clock)
        self._estimation = False
        self._compensation = False
        self.Ke = None
        self._Vrec = []
        self._Irec = []

    def start_injection(self, amp=.5 * nA, DC=0 * nA):
        '''
        Start white noise injection for kernel estimation.
        amp = current amplitude
        DC = additional DC current
        '''
        self._amp, self._DC = amp, DC
        self._estimation = True

    def stop_injection(self):
        '''
        Stop white noise injection.
        '''
        self._estimation = False
        self.command = 0 #*nA

    def estimate(self, ksize=150, ktail=50, dendritic=False):
        '''
        Estimate electrode kernel Ke (after injection)
        
        ksize = kernel size (in bins)
        ktail = tail parameter (in bins), indicates the end of the electrode kernel
        '''
        self._ksize = ksize
        self._ktail = ktail
        # Calculate Ke
        vrec = array(self._Vrec) / mV
        irec = array(self._Irec) / nA
        K = full_kernel(vrec, irec, self._ksize)
        self._K = K * Mohm
        if dendritic:
            self.Ke = electrode_kernel_dendrite(K, ktail) * Mohm
        else:
            self.Ke = electrode_kernel_soma(K, ktail) * Mohm
        self._Vrec = []
        self._Irec = []

    def switch_on(self, Ke=None):
        '''
        Switch compensation on, with kernel Ke.
        (If not given: use last kernel)
        '''
        self._compensation = True
        self._lastI = zeros(self._ktail)
        self._posI = 0

    def switch_off(self):
        '''
        Switch compensation off.
        '''
        self._compensation = False

    def update(self):
        AcquisitionBoard.update(self)
        if self._estimation:
            I = 2. * (rand() - .5) * self._amp + self._DC
            self.command = I
            # Record
            self._Vrec.append(self.record[0])
            self._Irec.append(I)
        if self._compensation:
            # Compensate
            self._lastI[self._posI] = self.command[0]
            self._posI = (self._posI - 1) % self._ktail
            self.record[0] = self.record[0] - sum(self.Ke * self._lastI[range(self._posI, self._ktail) +
                                                        range(0, self._posI)])


class VC_AEC(AEC):
    def __init__(self, P, V, I, gain=50 * nS, gain2=0 * nS / ms, clock=None):
        AEC.__init__(self, P, V, I, clock=clock)
        self._gain = gain
        self._gain2 = gain2
        self._J = zeros(len(P))

    def stop_injection(self):
        '''
        Stop white noise injection.
        '''
        self._estimation = False
        self.record = 0 #*nA

    def update(self):
        V = self._P.state(self._V) # Record
        self._P.state(self._I)[:] = -self.record # Inject
        if self._estimation:
            I = 2. * (rand() - .5) * self._amp + self._DC
            self.record = -I
            # Record
            self._Vrec.append(V[0])
            self._Irec.append(I)
        if self._compensation:
            # Compensate
            self._lastI[self._posI] = -self.record[0]
            self._posI = (self._posI - 1) % self._ktail
            V[0] = V[0] - sum(self.Ke * self._lastI[range(self._posI, self._ktail) + range(0, self._posI)])
            self._J += self.clock._dt * self._gain2 * (self.command - V)
            self.record = -(self._gain * (self.command - V) + self._J)

if __name__ == '__main__':
    from brian import *
    taum = 20 * ms
    gl = 20 * nS
    Cm = taum * gl
    eqs = Equations('dv/dt=(-gl*v+i_inj)/Cm : volt') + electrode(50 * Mohm, 10 * pF, vm='v', i_cmd=.5 * nA)
    neuron = NeuronGroup(1, model=eqs)
    M = StateMonitor(neuron, 'v_el', record=True)
    run(100 * ms)
    plot(M.times / ms, M[0] / mV)
    show()



"""
A ring of integrate-and-fire neurons.
"""
from brian import *

tau = 10 * ms
v0 = 11 * mV
N = 20
w = 1 * mV

ring = NeuronGroup(N, model='dv/dt=(v0-v)/tau : volt', threshold=10 * mV, reset=0 * mV)

W = Connection(ring, ring, 'v')
for i in range(N):
    W[i, (i + 1) % N] = w

ring.v = rand(N) * 10 * mV

S = SpikeMonitor(ring)

run(300 * ms)

raster_plot(S)
show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
'''
Monitors (spikes and state variables).
* Tip: Spike monitors should have non significant impact on simulation time
if properly coded.
'''

__all__ = ['VanRossumMetric','SpikeMonitor', 'PopulationSpikeCounter', 'SpikeCounter', 'FileSpikeMonitor', 'StateMonitor', 'ISIHistogramMonitor', 'Monitor',
           'PopulationRateMonitor', 'StateSpikeMonitor', 'MultiStateMonitor', 'RecentStateMonitor', 'CoincidenceCounter', 'CoincidenceMatrixCounter', 'StateHistogramMonitor']

from units import *
from connections import Connection, SparseConnectionVector
from numpy import array, zeros, mean, histogram, linspace, tile, digitize,     \
        copy, ones, rint, exp, arange, convolve, argsort, mod, floor, asarray, \
        maximum, Inf, amin, amax, sort, nonzero, setdiff1d, diag, hstack, resize,\
         inf, var,tril,empty,float64,array,sum
from scipy.spatial.distance import sqeuclidean
from itertools import repeat, izip
from clock import guess_clock, EventClock, Clock
from network import NetworkOperation, network_operation
from stdunits import ms, Hz
from collections import defaultdict
import types
from operator import isSequenceType
from tools.statistics import firing_rate
from neurongroup import NeuronGroup
import bisect
from base import *
from time import time
try:
    import pylab, matplotlib
except:
    pass


from globalprefs import *
from scipy import weave


class Monitor(object):
    pass


class SpikeMonitor(Connection, Monitor):
    '''
    Counts or records spikes from a :class:`NeuronGroup`

    Initialised as one of::
    
        SpikeMonitor(source(,record=True))
        SpikeMonitor(source,function=function)
    
    Where:
    
    ``source``
        A :class:`NeuronGroup` to record from
    ``record``
        ``True`` or ``False`` to record all the spikes or just summary
        statistics.
    ``function``
        A function ``f(spikes)`` which is passed the array of neuron
        numbers that have fired called each step, to define
        custom spike monitoring.
    
    Has three attributes:
    
    ``nspikes``
        The number of recorded spikes
    ``spikes``
        A time ordered list of pairs ``(i,t)`` where neuron ``i`` fired
        at time ``t``.
    ``spiketimes``
        A dictionary with keys the indices of the neurons, and values an
        array of the spike times of that neuron. For example,
        ``t=M.spiketimes[3]`` gives the spike times for neuron 3.

    For ``M`` a :class:`SpikeMonitor`, you can also write:
    
    ``M[i]``
        An array of the spike times of neuron ``i``.

    Notes:

    :class:`SpikeMonitor` is subclassed from :class:`Connection`.
    To define a custom monitor, either define a subclass and
    rewrite the ``propagate`` method, or pass the monitoring function
    as an argument (``function=myfunction``, with ``def myfunction(spikes):...``)
    '''
    # isn't there a units problem here for delay?
    def __init__(self, source, record=True, delay=0, function=None):
        # recordspikes > record?
        self.source = source # pointer to source group
        self.target = None
        self.nspikes = 0
        self.spikes = []
        self.record = record
        self.W = None # should we just remove this variable?
        source.set_max_delay(delay)
        self.delay = int(delay / source.clock.dt) # Synaptic delay in time bins
        self._newspikes = True
        if function != None:
            self.propagate = function

    def reinit(self):
        """
        Clears all monitored spikes
        """
        self.nspikes = 0
        self.spikes = []
        self._newspikes = True #recreate self._spiketimes on next access

    def propagate(self, spikes):
        '''
        Deals with the spikes.
        Overload this function to store or process spikes.
        Default: counts the spikes (variable nspikes)
        '''
        if len(spikes):
            self._newspikes = True
            self.nspikes += len(spikes)
            if self.record:
                self.spikes += zip(spikes, repeat(self.source.clock.t))

    def origin(self, P, Q):
        '''
        Returns the starting coordinate of the given groups in
        the connection matrix W.
        '''
        return (P.origin - self.source.origin, 0)

    def compress(self):
        pass

    def __getitem__(self, i):
        return self.getspiketimes()[i]

    def getspiketimes(self):
        if self._newspikes:
            self._newspikes = False
            self._spiketimes = {}
            for i in xrange(len(self.source)):
                self._spiketimes[i] = []
            for i, t in self.spikes:
                self._spiketimes[i].append(float(t))
            for i in xrange(len(self.source)):
                self._spiketimes[i] = array(self._spiketimes[i])
        return self._spiketimes
    spiketimes = property(fget=getspiketimes)

#    def getvspikes(self):
#        if isinstance(self.source, VectorizedNeuronGroup):
#            N = self.source.neuron_number
#            overlap = self.source.overlap
#            duration = self.source.duration
#            vspikes = [(mod(i,N),(t-overlap)+i/N*(duration-overlap)*second) for (i,t) in self.spikes if t >= overlap]
#            vspikes.sort(cmp=lambda x,y:2*int(x[1]>y[1])-1)
#            return vspikes
#    concatenated_spikes = property(fget=getvspikes)


class AutoCorrelogram(SpikeMonitor):
    '''
    Calculates autocorrelograms for the selected neurons (online).
    
    Initialised as::
    
        AutoCorrelogram(source,record=[1,2,3], delay=10*ms)
    
    where ``delay`` is the size of the autocorrelogram.
    
    NOT FINISHED 
    '''
    def __init__(self, source, record=True, delay=0):
        SpikeMonitor.__init__(self, source, record=record, delay=delay)
        self.reinit()
        if record is not False:
            if record is not True and not isinstance(record, int):
                self.recordindex = dict((i, j) for i, j in zip(self.record, range(len(self.record))))

    def reinit(self):
        if self.record == True:
            self._autocorrelogram = zeros((len(self.record), len(self.source)))
        else:
            self._autocorrelogram = zeros((len(self.record), self.delay))

    def propagate(self, spikes):
        spikes_set = set(spikes)
        if self.record == True:
            for i in xrange(self.delay): # Not a brilliant implementation
                self._autocorrelogram[spikes_set.intersection(self.source.LS[i]), i] += 1

    def __getitem__(self, i):
        # TODO: returns the autocorrelogram of neuron i
        pass


class PopulationSpikeCounter(SpikeMonitor):
    '''
    Counts spikes from a :class:`NeuronGroup`

    Initialised as::
    
        PopulationSpikeCounter(source)
    
    With argument:
    
    ``source``
        A :class:`NeuronGroup` to record from
    
    Has one attribute:
    
    ``nspikes``
        The number of recorded spikes
    '''
    def __init__(self, source, delay=0):
        SpikeMonitor.__init__(self, source, record=False, delay=delay)


class SpikeCounter(PopulationSpikeCounter):
    '''
    Counts spikes from a :class:`NeuronGroup`

    Initialised as::
    
        SpikeCounter(source)
    
    With argument:
    
    ``source``
        A :class:`NeuronGroup` to record from
    
    Has two attributes:
    
    ``nspikes``
        The number of recorded spikes
    ``count``
        An array of spike counts for each neuron
    
    For a :class:`SpikeCounter` ``M`` you can also write ``M[i]`` for the
    number of spikes counted for neuron ``i``.
    '''
    def __init__(self, source):
        PopulationSpikeCounter.__init__(self, source)
        self.count = zeros(len(source), dtype=int)

    def __getitem__(self, i):
        return int(self.count[i])

    def propagate(self, spikes):
        if len(spikes):
            PopulationSpikeCounter.propagate(self, spikes)
            self.count[spikes] += 1

    def reinit(self):
        self.count[:] = 0
        PopulationSpikeCounter.reinit(self)


class StateSpikeMonitor(SpikeMonitor):
    '''
    Counts or records spikes and state variables at spike times from a :class:`NeuronGroup`

    Initialised as::
    
        StateSpikeMonitor(source, var)
    
    Where:
    
    ``source``
        A :class:`NeuronGroup` to record from
    ``var``
        The variable name or number to record from, or a tuple of variable names or numbers
        if you want to record multiple variables for each spike.
    
    Has two attributes:
    
    .. attribute:: nspikes
    
        The number of recorded spikes
        
    .. attribute:: spikes
    
        A time ordered list of tuples ``(i,t,v)`` where neuron ``i`` fired
        at time ``t`` and the specified variable had value ``v``. If you
        specify multiple variables, each tuple will be of the form
        ``(i,t,v0,v1,v2,...)`` where the ``vi`` are the values corresponding
        in order to the variables you specified in the ``var`` keyword.
    
    And two methods:
    
    .. method:: times(i=None)
    
        Returns an array of the spike times for the whole monitored
        group, or just for neuron ``i`` if specified.
    
    .. method:: values(var, i=None)
    
        Returns an array of the values of variable ``var`` for the
        whole monitored group, or just for neuron ``i`` if specified.
    '''
    def __init__(self, source, var):
        SpikeMonitor.__init__(self, source)
        if isinstance(var, (str, int)) or not isSequenceType(var):
            var = (var,)
        self._varnames = var
        self._vars = [source.state_(v) for v in var]
        self._varindex = dict((v, i + 2) for i, v in enumerate(var))
        self._units = [source.unit(v) for v in var]

    def propagate(self, spikes):
        if len(spikes):
            self.nspikes += len(spikes)
            recordedstate = [ [x * u for x in v[spikes]] for v, u in izip(self._vars, self._units) ]
            self.spikes += zip(spikes, repeat(self.source.clock.t), *recordedstate)

    def __getitem__(self, i):
        return NotImplemented # don't use the version from SpikeMonitor

    def times(self, i=None):
        '''Returns the spike times (of neuron ``i`` if specified)'''
        if i is not None:
            return array([x[1] for x in self.spikes if x[0] == i])
        else:
            return array([x[1] for x in self.spikes])

    def values(self, var, i=None):
        '''Returns the recorded values of ``var`` (for spikes from neuron ``i`` if specified)'''
        v = self._varindex[var]
        if i is not None:
            return array([x[v] for x in self.spikes if x[0] == i])
        else:
            return array([x[v] for x in self.spikes])


class HistogramMonitorBase(SpikeMonitor):
    pass


class ISIHistogramMonitor(HistogramMonitorBase):
    '''
    Records the interspike interval histograms of a group.
    
    Initialised as::
    
        ISIHistogramMonitor(source, bins)
    
    ``source``
        The source group to record from.
    ``bins``
        The lower bounds for each bin, so that e.g.
        ``bins = [0*ms, 10*ms, 20*ms]`` would correspond to
        bins with intervals 0-10ms, 10-20ms and
        20+ms.
        
    Has properties:
    
    ``bins``
        The ``bins`` array passed at initialisation.
    ``count``
        An array of length ``len(bins)`` counting how many ISIs
        were in each bin.
    
    This object can be passed directly to the plotting function
    :func:`hist_plot`.
    '''
    def __init__(self, source, bins, delay=0):
        SpikeMonitor.__init__(self, source, delay)
        self.bins = array(bins)
        self.reinit()

    def reinit(self):
        super(ISIHistogramMonitor, self).reinit()
        self.count = zeros(len(self.bins))
        self.LS = 1000 * second * ones(len(self.source))

    def propagate(self, spikes):
        if len(spikes):
            super(ISIHistogramMonitor, self).propagate(spikes)
            isi = self.source.clock.t - self.LS[spikes]
            self.LS[spikes] = self.source.clock.t
            # all this nonsense is necessary to deal with the fact that
            # numpy changed the semantics of histogram in 1.2.0 or thereabouts
            try:
                h, a = histogram(isi, self.bins, new=True)
            except TypeError:
                h, a = histogram(isi, self.bins)
            if len(h) == len(self.count):
                self.count += h
            else:
                self.count[:-1] += h
                self.count[-1] += len(isi) - sum(h)


class FileSpikeMonitor(SpikeMonitor):
    """Records spikes to a file

    Initialised as::
    
        FileSpikeMonitor(source, filename[, record=False])
    
    Does everything that a :class:`SpikeMonitor` does except also records
    the spikes to the named file. note that spikes are recorded
    as an ASCII file of lines each of the form:
    
        ``i, t``
    
    Where ``i`` is the neuron that fired, and ``t`` is the time in seconds.
    
    Has one additional method:
    
    ``close_file()``
        Closes the file manually (will happen automatically when
        the program ends).
    """
    def __init__(self, source, filename, record=False, delay=0):
        super(FileSpikeMonitor, self).__init__(source, record, delay)
        self.filename = filename
        self.f = open(filename, 'w')

    def reinit(self):
        self.close_file()
        self.f = open(self.filename, 'w')

    def propagate(self, spikes):
        if len(spikes):
            super(FileSpikeMonitor, self).propagate(spikes)
            for i in spikes:
                self.f.write(str(i) + ", " + str(float(self.source.clock.t)) + "\n")

    def close_file(self):
        self.f.close()


class PopulationRateMonitor(SpikeMonitor):
    '''
    Monitors and stores the (time-varying) population rate
    
    Initialised as::
    
        PopulationRateMonitor(source,bin)
    
    Records the average activity of the group for every bin.
    
    Properties:
    
    ``rate``, ``rate_``
        An array of the rates in Hz.    
    ``times``, ``times_``
        The times of the bins.
    ``bin``
        The duration of a bin (in second).
    '''
    times = property(fget=lambda self:array(self._times))
    times_ = times
    rate = property(fget=lambda self:array(self._rate))
    rate_ = rate

    def __init__(self, source, bin=None):
        SpikeMonitor.__init__(self, source)
        if bin:
            self._bin = int(bin / source.clock.dt)
        else:
            self._bin = 1 # bin size in number
        self._rate = []
        self._times = []
        self._curstep = 0
        self._clock = source.clock
        self._factor = 1. / float(self._bin * source.clock.dt * len(source))

    def reinit(self):
        SpikeMonitor.reinit(self)
        self._rate = []
        self._times = []
        self._curstep = 0

    def propagate(self, spikes):
        if self._curstep == 0:
            self._rate.append(0.)
            self._times.append(self._clock._t) # +.5*bin?
            self._curstep = self._bin
        self._rate[-1] += len(spikes) * self._factor
        self._curstep -= 1

    def smooth_rate(self, width=None, filter='gaussian'):
        """
        Returns a smoothed version of the vector of rates,
        convolving the rates with a filter (gaussian or flat)
        with the given width.
        """
        if width is None: # automatic with Shinomoto's algorithms
            if filter=='flat':
                """ (Experimental)
                If width is not given and the filter is flat, then the bin
                size is automatically chosen using Shimazaki and Shinomoto's method:
                  Shimazaki and Shinomoto, A method for selecting the bin size of a time histogram
                  Neural Computation 19(6), 1503-1527, 2007
                  http://dx.doi.org/10.1162/neco.2007.19.6.1503
                """
                # Shinomoto's method to find the optimal bin size. Adapted from:
                # Shimazaki and Shinomoto, A method for selecting the bin size of a time histogram
                # Neural Computation 19(6), 1503-1527, 2007
                # http://dx.doi.org/10.1162/neco.2007.19.6.1503
                counts=array(self._rate)/self._factor
                best_value=inf
                for nbins in range(2,500): # possible number of bins (maybe a less brutal optimization?)
                    binsize=len(counts)/nbins
                    x=resize(counts,(len(counts)/binsize,binsize))
                    #x.reshape((x.size,1))[len(counts):]=0 # unnecessary because smaller
                    x=x.sum(1) # x is the histogram with nbins bins
                    K=mean(x) # average number of spikes per recording bin
                    value=(2*K-var(x))/binsize**2
                    if value<best_value:
                        best_value=value
                        width_dt=binsize
                        nb=nbins
                #print width_dt,nb
            else:
                raise AttributeError,"Automatic width selection is not implemented yet!"
        else:
            width_dt = int(width / (self._bin * self._clock.dt)) # width in number of bins
        #print width_dt
        window = {'gaussian': exp(-arange(-2 * width_dt, 2 * width_dt + 1) ** 2 * 1. / (2 * (width_dt) ** 2)),
                'flat': ones(width_dt)}[filter]
        return convolve(self.rate_, window * 1. / sum(window), mode='same')


class StateMonitor(NetworkOperation, Monitor):
    '''
    Records the values of a state variable from a :class:`NeuronGroup`.

    Initialise as::
    
        StateMonitor(P,varname(,record=False)
            (,when='end)(,timestep=1)(,clock=clock))
    
    Where:
    
    ``P``
        The group to be recorded from
    ``varname``
        The state variable name or number to be recorded
    ``record``
        What to record. The default value is ``False`` and the monitor will
        only record summary statistics for the variable. You can choose
        ``record=integer`` to record every value of the neuron with that
        number, ``record=``list of integers to record every value of each of
        those neurons, or ``record=True`` to record every value of every
        neuron (although beware that this may use a lot of memory).
    ``when``
        When the recording should be made in the network update, possible
        values are any of the strings: ``'start'``, ``'before_groups'``, ``'after_groups'``,
        ``'before_connections'``, ``'after_connections'``, ``'before_resets'``,
        ``'after_resets'``, ``'end'`` (in order of when they are run).
    ``timestep``
        A recording will be made each timestep clock updates (so ``timestep``
        should be an integer).
    ``clock``
        A clock for the update schedule, use this if you have specified a
        clock other than the default one in your network, or to update at a
        lower frequency than the update cycle. Note though that if the clock
        here is different from the main clock, the when parameter will not
        be taken into account, as network updates are done clock by clock.
        Use the ``timestep`` parameter if you need recordings to be made at a
        precise point in the network update step.

    The :class:`StateMonitor` object has the following properties:

    ``times``
        The times at which recordings were made
    ``mean``
        The mean value of the state variable for every neuron in the
        group (not just the ones specified in the ``record`` keyword)
    ``var``
        The unbiased estimate of the variances, as in ``mean``
    ``std``
        The square root of ``var``, as in ``mean``
    ``values``
        A 2D array of the values of all the recorded neurons, each row is a
        single neuron's values.
        
    In addition, if :class:`M`` is a :class:`StateMonitor` object, you write::
    
        M[i]
    
    for the recorded values of neuron ``i`` (if it was specified with the
    ``record`` keyword). It returns a numpy array.
    
    Methods:
    
    .. method:: plot([indices=None[, cmap=None[, refresh=None[, showlast=None[, redraw=True]]]]])
        
        Plots the recorded values using pylab. You can specify an index or
        list of indices, otherwise all the recorded values will be plotted.
        The graph plotted will have legends of the form ``name[i]`` for
        ``name`` the variable name, and ``i`` the neuron index. If cmap is
        specified then the colours will be set according to the matplotlib
        colormap cmap. ``refresh`` specifies how often (in simulation time)
        you would like the plot to refresh. Note that this will only work if
        pylab is in interactive mode, to ensure this call the pylab ``ion()``
        command. If you are using the ``refresh`` option, ``showlast`` specifies
        a fixed time window to display (e.g. the last 100ms).
        If you are using more than one realtime monitor, only one of them needs
        to issue a redraw command, therefore set ``redraw=False`` for all but
        one of them.
        
        Note that with some IDEs, interactive plotting will not work with the
        default matplotlib backend, try doing something like this at the
        beginning of your script (before importing brian)::
        
            import matplotlib
            matplotlib.use('WXAgg')
            
        You may need to experiment, try WXAgg, GTKAgg, QTAgg, TkAgg.
        
    .. method:: insert_spikes(spikemonitor[, value=0])

        Inserts spikes into recorded traces (for plotting). State values
        at spike times are replaced with the given value (peak value of spike).
    '''
    mean = property(fget=lambda self:self._mu / self.N)
    _mean = mean
    mean_ = _mean
    var = property(fget=lambda self:(self._sqr - self.N * self.mean_ ** 2) / (self.N - 1))
    var_ = var
    std = property(fget=lambda self:self.var ** .5)
    std_ = std
    times = property(fget=lambda self:array(self._times))
    times_ = times
    values = property(fget=lambda self:self.getvalues())
    values_ = values

    def __init__(self, P, varname, clock=None, record=False, timestep=1, when='end'):
        '''
        -- P is the neuron group
        -- varname is the variable name
        -- record can be one of:
           - an integer, in which case the value of the state of the corresponding
           neuron will be recorded in the list self._values
           - an array or list of integers, in which case the value of the states
           of the corresponding neurons will be recorded and can be individually
           accessed by calling self[i] where i is the neuron number
           - True, in which case the state of all neurons is recorded, and can be
           individually accessed by calling self[i]
           - False, in which case only the mean and variance are recorded (.mean, .var, .std)
        -- timestep defines how often a recording is made (e.g. if you have a very
           small dt, you might not want to record every value of the variable), it
           is an integer (multiple of the clock dt)
        '''
        NetworkOperation.__init__(self, None, clock=clock, when=when)
        self.record = record
        self.clock = guess_clock(clock)
        if record is not False:
            if record is not True and not isinstance(record, int):
                self.recordindex = dict((i, j) for i, j in zip(self.record, range(len(self.record))))
        self.timestep = timestep
        self.curtimestep = timestep
        self._values = None
        self.P = P
        self.varname = varname
        self.N = 0 # number of steps
        self._recordstep = 0
        if record is False:
            self._mu = zeros(len(P)) # sum
            self._sqr = zeros(len(P)) # sum of squares
        self.unit = 1.0 * P.unit(varname)
        self.reinit()

    def __call__(self):
        '''
        This function is called every time step.
        '''
        V = self.P.state_(self.varname)
        if self.record is False:
            self._mu += V
            self._sqr += V * V
        elif self.curtimestep == self.timestep:
            i = self._recordstep
            if not isinstance(self.record, bool):
                self._values.append(V[self.record])
            elif self.record is True:
                self._values.append(V.copy())
            self._times.append(self.clock._t)
            self._recordstep += 1
        self.curtimestep -= 1
        if self.curtimestep == 0: self.curtimestep = self.timestep
        self.N += 1

    def __getitem__(self, i):
        """Returns the recorded values of the state of neuron i as an array
        """
        if self.record is False:
            raise IndexError('Neuron ' + str(i) + ' was not recorded.')
        if self.record is not True:
            if isinstance(self.record, int) and self.record != i or (not isinstance(self.record, int) and i not in self.record):
                raise IndexError('Neuron ' + str(i) + ' was not recorded.')
            try:
                return self.values[self.recordindex[i]]
            except:
                if i == self.record:
                    return self.values[0]
                else:
                    raise
        elif self.record is True:
            return self.values[i]

    def getvalues(self):
        if len(self._values):
            newvalues = array(self._values)
            if len(newvalues.shape)==1:
                newvalues.shape = (1, newvalues.size)
            else:
                newvalues = newvalues.T
            values = hstack((self._values_cache, newvalues))
            self._values_cache = values
            self._values = []
        else:
            values = self._values_cache    
        return values
    getvalues_ = getvalues

    def reinit(self):
        self._values = []
        self._times = []
        ri = self.get_record_indices()
        self._values_cache = zeros((len(ri), 0))
        self.N = 0
        self._recordstep = 0
        self._mu = zeros(len(self.P))
        self._sqr = zeros(len(self.P))

    def get_record_indices(self):
        """Returns the list of neuron numbers which were recorded.
        """
        if self.record is False:
            return []
        elif self.record is True:
            return arange(len(self.P))
        elif isinstance(self.record, int):
            return [self.record]
        else:
            return self.record

    def plot(self, indices=None, cmap=None, refresh=None, showlast=None, redraw=True):
        lines = []
        inds = []
        if indices is None:
            recind = self.get_record_indices()
            for j, i in enumerate(recind):
                if cmap is None:
                    line, = pylab.plot(self.times, self[i], label=str(self.varname) + '[' + str(i) + ']')
                else:
                    line, = pylab.plot(self.times, self[i], label=str(self.varname) + '[' + str(i) + ']',
                               color=cmap(float(j) / (len(recind) - 1)))
                inds.append(i)
                lines.append(line)
        elif isinstance(indices, int):
            line, = pylab.plot(self.times, self[indices], label=str(self.varname) + '[' + str(indices) + ']')
            lines.append(line)
            inds.append(indices)
        else:
            for j, i in enumerate(indices):
                if cmap is None:
                    line, = pylab.plot(self.times, self[i], label=str(self.varname) + '[' + str(i) + ']')
                else:
                    line, = pylab.plot(self.times, self[i], label=str(self.varname) + '[' + str(i) + ']',
                               color=cmap(float(j) / (len(indices) - 1)))
                inds.append(i)
                lines.append(line)
        ax = pylab.gca()
        if refresh is not None:
            ylim = [Inf, -Inf]
            @network_operation(clock=EventClock(dt=refresh))
            def refresh_state_monitor_plot(clk):
                if self.times.size==0: # bugfix submitted by Oleg Sinyavskiy
                    return
                ymin, ymax = ylim
                if matplotlib.is_interactive():
                    if showlast is not None:
                        tmin = clk._t - float(showlast)
                        tmax = clk._t
                    for line, i in zip(lines, inds):
                        if showlast is None:
                            line.set_xdata(self.times)
                            y = self[i]
                        else:
                            imin = bisect.bisect_left(self.times, tmin)
                            imax = bisect.bisect_right(self.times, tmax)
                            line.set_xdata(self.times[imin:imax])
                            y = self[i][imin:imax]
                        line.set_ydata(y)
                        ymin = min(ymin, amin(y))
                        ymax = max(ymax, amax(y))
                    if showlast is None:
                        ax.set_xlim(0, clk._t)
                    else:
                        ax.set_xlim(clk._t - float(showlast), clk._t)
                    ax.set_ylim(ymin, ymax)
                    ylim[:] = [ymin, ymax]
                    if redraw:
                        pylab.draw()
                        pylab.get_current_fig_manager().canvas.flush_events()
            self.contained_objects.append(refresh_state_monitor_plot)

    def insert_spikes(self, spikemonitor, value=0):
        """
        Inserts spikes into recorded traces (for plotting). State values
        at spike times are replaced with the given value (peak value of spike).
        """
        dt = self.clock.dt
        values = self.values
        for i, neuron in enumerate(self.get_record_indices()):
            values[i,array(spikemonitor[neuron]/dt, dtype=int)] = value
        #self._values = values # or converted back to a list?


class RecentStateMonitor(StateMonitor):
    '''
    StateMonitor that records only the most recent fixed amount of time.
    
    Works in the same way as a :class:`StateMonitor` except that it has one
    additional initialiser keyword ``duration`` which gives the length of
    time to record values for, the ``record`` keyword defaults to ``True``
    instead of ``False``, and there are some different or additional
    attributes:
    
    ``values``, ``values_``, ``times``, ``times_``
        These will now return at most the most recent values over an
        interval of maximum time ``duration``. These arrays are copies,
        so for faster access use ``unsorted_values``, etc.
    ``unsorted_values``, ``unsorted_values_``, ``unsorted_times``, ``unsorted_times_``
        The raw versions of the data, the associated times may not be
        in sorted order and if ``duration`` hasn't passed, not all the
        values will be meaningful.
    ``current_time_index``
        Says which time index the next values to be recorded will be stored
        in, varies from 0 to M-1.
    ``has_looped``
        Whether or not the ``current_time_index`` has looped from M back to
        0 - can be used to tell whether or not every value in the
        ``unsorted_values`` array is meaningful or not (they will only all
        be meaningful when ``has_looped==True``, i.e. after time ``duration``).
    
    The ``__getitem__`` method also returns values in sorted order.
    
    To plot, do something like::
    
        plot(M.times, M.values[:, i])
    '''
    def __init__(self, P, varname, duration=5 * ms, clock=None, record=True, timestep=1, when='end'):
        StateMonitor.__init__(self, P, varname, clock=clock, record=record, timestep=timestep, when=when)
        self.duration = duration
        self.num_duration = int(duration / (timestep * self.clock.dt)) + 1
        if record is False:
            self.record_size = 0
        elif record is True:
            self.record_size = len(P)
        elif isinstance(record, int):
            self.record_size = 1
        else:
            self.record_size = len(record)
        self._values = zeros((self.num_duration, self.record_size))
        self._times = zeros(self.num_duration)
        self.current_time_index = 0
        self.has_looped = False
        self._invtargetdt = 1.0 / self.clock._dt
        self._arange = arange(len(P))

    def __call__(self):
        V = self.P.state_(self.varname)
        if self.record is False:
            self._mu += V
            self._sqr += V * V
        if self.record is not False and self.curtimestep == self.timestep:
            i = self._recordstep
            if self.record is not True:
                self._values[self.current_time_index, :] = V[self.record]
            else:
                self._values[self.current_time_index, :] = V
            self._times[self.current_time_index] = self.clock.t
            self._recordstep += 1
            self.current_time_index = (self.current_time_index + 1) % self.num_duration
            if self.current_time_index == 0: self.has_looped = True
        self.curtimestep -= 1
        if self.curtimestep == 0: self.curtimestep = self.timestep
        self.N += 1

    def __getitem__(self, i):
        timeinds = self.sorted_times_indices()
        if self.record is False:
            raise IndexError('Neuron ' + str(i) + ' was not recorded.')
        if self.record is not True:
            if isinstance(self.record, int) and self.record != i or (not isinstance(self.record, int) and i not in self.record):
                raise IndexError('Neuron ' + str(i) + ' was not recorded.')
            try:
                return self._values[timeinds, self.recordindex[i]]
            except:
                if i == self.record:
                    return self._values[timeinds, 0]
                else:
                    raise
        elif self.record is True:
            return self._values[timeinds, i]

    def get_past_values(self, times):
        # probably mostly to be used internally by Brian itself
        time_indices = (self.current_time_index - 1 - array(self._invtargetdt * asarray(times), dtype=int)) % self.num_duration
        if isinstance(times, SparseConnectionVector):
            return SparseConnectionVector(times.n, times.ind, self._values[time_indices, times.ind])
        else:
            return self._values[time_indices, self._arange]

    def get_past_values_sequence(self, times_seq):
        # probably mostly to be used internally by Brian itself
        if len(times_seq) == 0:
            return []
        time_indices_seq = [(self.current_time_index - 1 - array(self._invtargetdt * asarray(times), dtype=int)) % self.num_duration for times in times_seq]
        if isinstance(times_seq[0], SparseConnectionVector):
            return [SparseConnectionVector(times.n, times.ind, self._values[time_indices, times.ind]) for times, time_indices in izip(times_seq, time_indices_seq)]
        else:
            return [self._values[time_indices, self._arange] for times, time_indices in izip(times_seq, time_indices_seq)]

    def getvalues(self):
        return self._values
    getvalues_ = getvalues

    def sorted_times_indices(self):
        if not self.has_looped:
            return arange(self.current_time_index)
        return argsort(self._times)

    def get_sorted_times(self):
        return self._times[self.sorted_times_indices()]
    get_sorted_times_ = get_sorted_times

    def get_sorted_values(self):
        return self._values[self.sorted_times_indices(), :]
    get_sorted_values_ = get_sorted_values

    times = property(fget=get_sorted_times)
    times_ = property(fget=get_sorted_times_)
    values = property(fget=get_sorted_values)
    values_ = property(fget=get_sorted_values_)
    unsorted_times = property(fget=lambda self:array(self._times))
    unsorted_times_ = unsorted_times
    unsorted_values = property(fget=getvalues)
    unsorted_values_ = unsorted_values

    def reinit(self):
        # We check self._values is not None because the __init__ of this class
        # calls the __init__ of StateMonitor which calls reinit, but this happens
        # before self._values is set to be an array.
        if self._values is not None:
            self._values[:] = 0
            self._times[:] = 0
        self.current_time_index = 0
        self.N = 0
        self._recordstep = 0
        self._mu = zeros(len(self.P))
        self._sqr = zeros(len(self.P))
        self.has_looped = False

    def plot(self, indices=None, cmap=None, refresh=None, showlast=None, redraw=True):
        if refresh is not None and showlast is None:
            showlast = self.duration
        StateMonitor.plot(self, indices=indices, cmap=cmap, refresh=refresh, showlast=showlast, redraw=redraw)


class MultiStateMonitor(NetworkOperation):
    '''
    Monitors multiple state variables of a group
    
    This class is a container for multiple :class:`StateMonitor` objects,
    one for each variable in the group. You can retrieve individual
    :class:`StateMonitor` objects using ``M[name]`` or retrieve the
    recorded values using ``M[name, i]`` for neuron ``i``.

    Initialised with a group ``G`` and a list of variables ``vars``. If 
    ``vars`` is omitted then all the variables of ``G`` will be recorded.
    Any additional keyword argument used to initialise the object will
    be passed to the individual :class:`StateMonitor` objects (e.g. the
    ``when`` keyword).
    
    Methods:
    
    ``items()``, ``iteritems()``
        Returns the pairs (var, mon)
    ``plot([indices[, cmap]])``
        Plots all the monitors (note that real-time plotting is not supported
        for this class).
    
    Attributes:

    ``vars``
        The list of variables recorded.
    ``times``
        The times at which recordings were made.
    ``monitors``
        The dictionary of monitors indexed by variable name.
    
    Usage::
        
        G = NeuronGroup(N, eqs, ...)
        M = MultiStateMonitor(G, record=True)
        ...
        run(...)
        ...
        plot(M['V'].times, M['V'][0])
        figure()
        for name, m in M.iteritems():
            plot(m.times, m[0], label=name)
        legend()
        show()
    '''
    def __init__(self, G, vars=None, clock=None, **kwds):
        NetworkOperation.__init__(self, lambda : None, clock=clock)
        self.monitors = {}
        if vars is None:
            vars = [name for name in G.var_index.keys() if isinstance(name, str)]
        self.vars = vars
        for varname in vars:
            self.monitors[varname] = StateMonitor(G, varname, clock=clock, **kwds)
        self.contained_objects = self.monitors.values()

    def __getitem__(self, varname):
        if isinstance(varname, tuple):
            varname, i = varname
            return self.monitors[varname][i]
        else:
            return self.monitors[varname]

    def vars(self):
        return self.monitors.keys()

    def iteritems(self):
        return self.monitors.iteritems()

    def items(self):
        return self.monitors.items()

    def plot(self, indices=None, cmap=None):
        for k, m in self.monitors.iteritems():
            m.plot(indices, cmap=cmap)

    def get_times(self):
        return self.monitors.values()[0].times

    times = property(fget=lambda self:self.get_times())

    def __call__(self):
        pass


class CoincidenceCounter(SpikeMonitor):
    """
    Coincidence counter class.
    
    Counts the number of coincidences between the spikes of the neurons in the network (model spikes),
    and some user-specified data spike trains (target spikes). This number is defined as the number of 
    target spikes such that there is at least one model spike within +- ``delta``, where ``delta``
    is the half-width of the time window.
    
    Initialised as::
    
        cc = CoincidenceCounter(source, data, delta = 4*ms)
    
    with the following arguments:
    
    ``source``
        A :class:`NeuronGroup` object which neurons are being monitored.
    
    ``data``
        The list of spike times. Several spike trains can be passed in the following way.
        Define a single 1D array ``data`` which contains all the target spike times one after the
        other. Now define an array ``spiketimes_offset`` of integers so that neuron ``i`` should 
        be linked to target train: ``data[spiketimes_offset[i]], data[spiketimes_offset[i]+1]``, etc.
        
        It is essential that each spike train with the spiketimes array should begin with a spike at a
        large negative time (e.g. -1*second) and end with a spike that is a long time
        after the duration of the run (e.g. duration+1*second).
    
    ``delta=4*ms``
        The half-width of the time window for the coincidence counting algorithm.
    
    ``spiketimes_offset``
        A 1D array, ``spiketimes_offset[i]`` is the index of the first spike of 
        the target train associated to neuron i.
        
    ``spikedelays``
        A 1D array with spike delays for each neuron. All spikes from the target 
        train associated to neuron i are shifted by ``spikedelays[i]``.
        
    ``coincidence_count_algorithm``
        If set to ``exclusive``, the algorithm cannot count more than one
        coincidence for each model spike.
        If set to ``inclusive``, the algorithm can count several coincidences
        for a single model spike.
    
    ``onset``
        A scalar value in seconds giving the start of the counting: no
        coincidences are counted before ``onset``.
    
    Has three attributes:
    
    ``coincidences``
        The number of coincidences for each neuron of the :class:`NeuronGroup`.
        ``coincidences[i]`` is the number of coincidences for neuron i.
        
    ``model_length``
        The number of spikes for each neuron. ``model_length[i]`` is the spike
        count for neuron i.
        
    ``target_length``
        The number of spikes in the target spike train associated to each neuron.
    """
    def __init__(self, source, data=None, spiketimes_offset=None, spikedelays=None,
                 coincidence_count_algorithm='exclusive', onset=None, delta=4 * ms):

        source.set_max_delay(0)
        self.source = source
        self.delay = 0
        if onset is None:
            onset = 0 * ms
        self.onset = onset
        self.N = len(source)
        self.coincidence_count_algorithm = coincidence_count_algorithm

        self.data = array(data)
        if spiketimes_offset is None:
            spiketimes_offset = zeros(self.N, dtype='int')
        self.spiketimes_offset = array(spiketimes_offset)

        if spikedelays is None:
            spikedelays = zeros(self.N)
        self.spikedelays = array(spikedelays)

        dt = self.source.clock.dt
        self.delta = int(rint(delta / dt))
        self.reinit()

    def reinit(self):
        dt = self.source.clock.dt
        # Number of spikes for each neuron
        self.model_length = zeros(self.N, dtype='int')
        self.target_length = zeros(self.N, dtype='int')

        self.coincidences = zeros(self.N, dtype='int')
        self.spiketime_index = self.spiketimes_offset
        self.last_spike_time = array(rint(self.data[self.spiketime_index] / dt), dtype=int)
        self.next_spike_time = array(rint(self.data[self.spiketime_index + 1] / dt), dtype=int)

        # First target spikes (needed for the computation of 
        #   the target train firing rates)
        self.first_target_spike = zeros(self.N)

        self.last_spike_allowed = ones(self.N, dtype='bool')
        self.next_spike_allowed = ones(self.N, dtype='bool')

    def propagate(self, spiking_neurons):
        dt = self.source.clock.dt
        #T = array(rint((self.source.clock.t + self.spikedelays)/dt), dtype = int)
        spiking_neurons = array(spiking_neurons)
        if len(spiking_neurons):

            if self.source.clock.t >= self.onset:
                self.model_length[spiking_neurons] += 1

            T_spiking = array(rint((self.source.clock.t + self.spikedelays[spiking_neurons]) / dt), dtype=int)

            remaining_neurons = spiking_neurons
            remaining_T_spiking = T_spiking
            while True:
                remaining_indices, = (remaining_T_spiking > self.next_spike_time[remaining_neurons]).nonzero()
                if len(remaining_indices):
                    indices = remaining_neurons[remaining_indices]
                    self.target_length[indices] += 1
                    self.spiketime_index[indices] += 1
                    self.last_spike_time[indices] = self.next_spike_time[indices]
                    self.next_spike_time[indices] = array(rint(self.data[self.spiketime_index[indices] + 1] / dt), dtype=int)
                    if self.coincidence_count_algorithm == 'exclusive':
                        self.last_spike_allowed[indices] = self.next_spike_allowed[indices]
                        self.next_spike_allowed[indices] = True
                    remaining_neurons = remaining_neurons[remaining_indices]
                    remaining_T_spiking = remaining_T_spiking[remaining_indices]
                else:
                    break

            # Updates coincidences count
            near_last_spike = self.last_spike_time[spiking_neurons] + self.delta >= T_spiking
            near_next_spike = self.next_spike_time[spiking_neurons] - self.delta <= T_spiking
            last_spike_allowed = self.last_spike_allowed[spiking_neurons]
            next_spike_allowed = self.next_spike_allowed[spiking_neurons]
            I = (near_last_spike & last_spike_allowed) | (near_next_spike & next_spike_allowed)

            if self.source.clock.t >= self.onset:
                self.coincidences[spiking_neurons[I]] += 1

            if self.coincidence_count_algorithm == 'exclusive':
                near_both_allowed = (near_last_spike & last_spike_allowed) & (near_next_spike & next_spike_allowed)
                self.last_spike_allowed[spiking_neurons] = last_spike_allowed & -near_last_spike
                self.next_spike_allowed[spiking_neurons] = (next_spike_allowed & -near_next_spike) | near_both_allowed

class VanRossumMetric(StateMonitor):
    """
    van Rossum spike train metric.
    From M. van Rossum (2001): A novel spike distance (Neural Computation). 
    
    Compute the van Rossum distance between every spike train from the source
    population.
    
    Arguments:
    
    ``source``
        The group to compute the distances for. 
    ``tau``
        Time constant of the kernel (low pass filter).
    
    Has one attribute:
    
    ``distance``
        A square symmetric matrix containing the distances.    
    """
    def __init__(self, source, tau=2*ms):
        self.dt = source.clock.dt
        self.source = source
        self.nbr_neurons = len(source)
        self.tau=tau

        eqs="""
        dv/dt=(-v)/tau: volt
        """
        kernel=NeuronGroup(self.nbr_neurons,model=eqs)
        C = Connection(source, kernel, 'v')
        C.connect_one_to_one(source,kernel)
        StateMonitor.__init__(self,kernel, 'v', record=True)
        self.contained_objects=[kernel,C]
        #self.distance_matrix=zeros((self.nbr_neurons,self.nbr_neurons))  
               
    def reinit(self):
        StateMonitor.reinit(self)
        
#    def define(self):
#        @network_operation(clock=EventClock(dt=self.dt))
#        def get_distance_online():
#            tt=time()   
#            for neuron_idx1 in range(self.nbr_neurons):
#                for neuron_idx2 in range((neuron_idx1+1)):
#                    self.distance_matrix[neuron_idx1,neuron_idx2]=self.distance_matrix[neuron_idx1,neuron_idx2]+self.dt/self.tau*abs(self[neuron_idx1][-1]-self[neuron_idx2][-1])**2
#            print time()-tt
#        self.contained_objects.append(get_distance_online)
        
    def get_distance(self):

        if get_global_preference('useweave'):

            _cpp_compiler=get_global_preference('weavecompiler')
            _extra_compile_args=['-O3']
            if _cpp_compiler=='gcc':
                _extra_compile_args+=get_global_preference('gcc_options') # ['-march=native', '-ffast-math']        
            nbr_neurons=int(self.nbr_neurons)
            distance_matrix=zeros((nbr_neurons,nbr_neurons),dtype=float64)
            nbr_time_step=int(len(self[0]))
            dt=float(self.dt)
            traces=self.values
            tau=float(self.tau)
            
            code='''
            for(int k1=0;k1<nbr_neurons;k1++)
            {
                for(int k2=0;k2<k1;k2++)
                {
                    double &dm = distance_matrix[k1*nbr_neurons+k2];
                    double *tr1 = traces+k1*nbr_time_step;
                    double *tr2 = traces+k2*nbr_time_step;
                    for(int istep=0;istep<nbr_time_step;istep++, tr1++, tr2++)
                    {
                        double diff = *tr1-*tr2;
                        dm += diff*diff; 
                    }
                    dm *= dt/tau;
                }
            }
            '''
            tt=time() 
            weave.inline(code, ['nbr_time_step','nbr_neurons','dt','tau','distance_matrix','traces'],
                         compiler=_cpp_compiler,
                         extra_compile_args=_extra_compile_args)
            print time()-tt
            return tril(distance_matrix,k=0)+tril(distance_matrix,k=0).T
        else:
            nbr_time_step=int(len(self[0]))
            self.distance_matrix=zeros((self.nbr_neurons,self.nbr_neurons))
            values = self.values
            memsize_mb = float(self.nbr_neurons*nbr_time_step*8)/1024**2
           # tt=time()
            if memsize_mb>200:
                for neuron_idx1 in xrange(self.nbr_neurons):
                    vidx1 = values[neuron_idx1]
                    for neuron_idx2 in xrange((neuron_idx1+1)):
                        self.distance_matrix[neuron_idx1,neuron_idx2]=self.dt/self.tau*sum((vidx1-values[neuron_idx2])**2)
            else:
                for neuron_idx1 in xrange(self.nbr_neurons):
                    Vi = values[neuron_idx1].reshape((1, nbr_time_step))
                    Vj = values.reshape((self.nbr_neurons, nbr_time_step))
                    self.distance_matrix[neuron_idx1, :] = (self.dt/self.tau)*sum((Vi-Vj)**2, axis=1)                
            #print time()-tt

            return tril(self.distance_matrix,k=0)+tril(self.distance_matrix,k=0).T

            
    distance = property(fget=get_distance)



class CoincidenceMatrixCounter(SpikeMonitor):
    """ 
    Coincidence counter matrix class.
    
    Counts the number of coincidences between the spikes of the neurons in the network (model spikes). This yields a matrix
    with the coincidence counts between every pair of neurons in the network
    
    Initialised as::
    
        cc = CoincidenceCounter(source, delta = 4*ms)
    
    with the following arguments:
    
    ``source``
        A :class:`NeuronGroup` object which neurons are being monitored.
        
    ``delta=4*ms``
        The half-width of the time window for the coincidence counting algorithm.      

    ``onset``
        A scalar value in seconds giving the start of the counting: no
        coincidences are counted before ``onset``.
    
    Has three attributes:
    
    ``coincidences``
        The matrix containg the number of coincidences between each neuron of the :class:`NeuronGroup`.
        ``coincidences[i,j]`` is the number of coincidences between neuron i and j.
        
    ``model_length``
        The number of spikes for each neuron. ``model_length[i]`` is the spike
        count for neuron i.
    
    """
    def __init__(self, source, onset=None, delta=4 * ms):

        source.set_max_delay(0)
        self.source = source
        self.delay = 0
        if onset is None:
            onset = 0 * ms
        self.onset = onset
        self.N = len(source)

        dt = self.source.clock.dt
        self.delta = array(rint(delta / dt), dtype=int)
        self.reinit()

    def reinit(self):
        dt = self.source.clock.dt # does not seem to be used
        # Number of spikes for each neuron
        self.model_length = zeros(self.N, dtype='int')
        self.target_length = zeros(self.N, dtype='int')

        self._coincidences = zeros((self.N, self.N), dtype='int')
        self.last_spike_time = -100 * ones(self.N, dtype='int')

    def get_coincidences(self):
        M = array(self._coincidences, dtype=float)
        M -= diag(M.diagonal() / 2)
        return M

    coincidences = property(fget=get_coincidences)

    def propagate(self, spiking_neurons):
        dt = self.source.clock.dt
        spiking_neurons = array(spiking_neurons)
        if len(spiking_neurons):
            if self.source.clock.t >= self.onset:
                self.model_length[spiking_neurons] += 1
            tint = array(rint(self.source.clock.t / dt), dtype=int)
            self.last_spike_time[spiking_neurons] = tint

            I, = (abs(self.last_spike_time - tint) <= self.delta).nonzero()
            if self.source.clock.t >= self.onset:
                for ispike in spiking_neurons:
                    #self.coincidences[ispike,setdiff1d(I,ispike)]+=1
                    self._coincidences[ispike, I[ispike <= I]] += 1
                    self._coincidences[I[ispike <= I], ispike] += 1
#        if self.source.clock.t == self.onset:
#            self.coincidences=(self.coincidences+self.coincidences.T)/2


class StateHistogramMonitor(NetworkOperation, Monitor):
    '''
    Records the histogram of a state variable from a :class:`NeuronGroup`.

    Initialise as::
    
        StateHistogramMonitor(P,varname,range(,period=1*ms)(,nbins=20))
    
    Where:
    
    ``P``
        The group to be recorded from
    ``varname``
        The state variable name or number to be recorded
    ``range``
        The minimum and maximum values for the state variable. A 2-tuple of floats.
    ``period``
        When to record.
    ``nbins``
        Number of bins for the histogram.

    The :class:`StateHistogramMonitor` object has the following properties:

    ``mean``
        The mean value of the state variable for every neuron in the
        group
    ``var``
        The unbiased estimate of the variances, as in ``mean``
    ``std``
        The square root of ``var``, as in ``mean``
    ``hist``
        A 2D array of the histogram values of all the neurons, each row is a
        single neuron's histogram.
    ``bins``
        A 1D array of the bin centers used to compute the histogram
    ``bin_edges``
        A 1D array of the bin edges used to compute the histogram
        
    In addition, if :class:`M`` is a :class:`StateHistogramMonitor` object, you write::
    
        M[i]
    
    for the histogram of neuron ``i``.
    '''
    def __init__(self, group, varname, range, period=1 * ms, nbins=20):
        self.clock = Clock(period)
        NetworkOperation.__init__(self, None, clock=self.clock)
        self.group = group
        self.len = len(group)
        self.varname = varname
        self.nbins = nbins
        self.n = 0
        self.bin_edges = linspace(range[0], range[1], self.nbins + 1)
        self._hist = zeros((self.len, self.nbins + 2))
        self.sum = zeros(self.len)
        self.sum2 = zeros(self.len)

    def __call__(self):
        x = self.group.state_(self.varname)
        self.sum += x
        self.sum2 += x ** 2
        inds = digitize(x, self.bin_edges)
        for i in xrange(self.len):
            self._hist[i, inds[i]] += 1
        self.n += 1

    def __getitem__(self, i):
        return self.hist[i, :]

    hist = property(fget=lambda self:self._hist[:, 1:-1] / self.n)
    bins = property(fget=lambda self:(self.bin_edges[:-1] + self.bin_edges[1:]) / 2)

    mean = property(fget=lambda self:self.sum / self.n)
    var = property(fget=lambda self:(self.sum2 / self.n - self.mean ** 2))
    std = property(fget=lambda self:self.var ** .5)



'''
An adaptive neuron model
'''
from brian import *

PG = PoissonGroup(1, 500 * Hz)
eqs = '''
dv/dt = (-w-v)/(10*ms) : volt # the membrane equation
dw/dt = -w/(30*ms) : volt # the adaptation current
'''
# The adaptation variable increases with each spike
IF = NeuronGroup(1, model=eqs, threshold=20 * mV,
                 reset='''v  = 0*mV
                          w += 3*mV ''')

C = Connection(PG, IF, 'v', weight=3 * mV)

MS = SpikeMonitor(PG, True)
Mv = StateMonitor(IF, 'v', record=True)
Mw = StateMonitor(IF, 'w', record=True)

run(100 * ms)

plot(Mv.times / ms, Mv[0] / mV)
plot(Mw.times / ms, Mw[0] / mV)

show()



# ----------------------------------------------------------------------------------
# Copyright ENS, INRIA, CNRS
# Contributors: Romain Brette (brette@di.ens.fr) and Dan Goodman (goodman@di.ens.fr)
# 
# Brian is a computer program whose purpose is to simulate models
# of biological neural networks.
# 
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 
# 
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 
# 
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
# ----------------------------------------------------------------------------------
# 
######### PHYSICAL UNIT NAMES #####################
#------------------------------------ Dan Goodman -
# These are optional shorthand unit names which in
# most circumstances shouldn't clash with local names

"""Optional short unit names

This module defines the following short unit names:

mV, mA, uA (micro_amp), nA, pA, mF, uF, nF, mS, uS, ms,
Hz, kHz, MHz, cm, cm2, cm3, mm, mm2, mm3, um, um2, um3
"""

import units as _units

mV = _units.mvolt

mA = _units.mamp
uA = _units.uamp
nA = _units.namp
pA = _units.pamp

pF = _units.pfarad
uF = _units.ufarad
nF = _units.nfarad

nS = _units.nsiemens
uS = _units.usiemens

ms = _units.msecond

Hz = _units.hertz
kHz = _units.khertz
MHz = _units.Mhertz

cm = _units.cmetre
cm2 = _units.cmetre2
cm3 = _units.cmetre3
mm = _units.mmetre
mm2 = _units.mmetre2
mm3 = _units.mmetre3
um = _units.umetre
um2 = _units.umetre2
um3 = _units.umetre3

_units.all_units.extend([mV, mA, uA, nA, pA, pF, uF, nF, nS, uS, ms, Hz, kHz, MHz, cm, cm2, cm3, mm, mm2, mm3, um, um2, um3])



'''
Input/output utility functions
'''
import numpy as np

__all__ = ['read_neuron_dat', 'read_atf']

def read_neuron_dat(name):
    '''
    Reads a Neuron vector file (.dat).
    
    Returns vector of times, vector of values
    '''
    f = open(name)
    f.readline(), f.readline() # skip first two lines
    M = np.loadtxt(f)
    f.close()
    return M[:, 0], M[:, 1]

def read_atf(name):
    '''
    Reads an Axon ATF file (.atf).
    
    Returns vector of times, vector of values
    '''
    f = open(name)
    f.readline()
    n = int(f.readline().split()[0]) # skip first two lines
    for _ in range(n + 1):
        f.readline()
    M = np.loadtxt(f)
    f.close()
    return M[:, 0], M[:, 1]

if __name__ == '__main__':
    from pylab import *
    t, v = read_neuron_dat(r"D:\My Dropbox\Neuron\Hu\recordings_Ifluct\I0_05_std_1_tau_10_sampling_100\vs.dat")
    #t,v=read_atf(r"/home/bertrand/Data/Measurements/Anna/input_file.atf")
    plot(t[:50000], v[:50000])
    show()



import os, sys, brian, re, datetime

def setreleasedate():
    releasedate = str(datetime.date.today())
    pathname, filename = os.path.split(__file__)
    os.chdir(pathname)
    os.chdir('../../../')
    # update __init__.py
    init_py = open('brian/__init__.py', 'r').read()
    init_py = re.sub("__release_date__\s*=\s*'.*?'", "__release_date__ = '" + releasedate + "'", init_py)
    open('brian/__init__.py', 'w').write(init_py)

if __name__ == '__main__':
    setreleasedate()



from numpy import *
from brian.library.modelfitting.clustertools import *


class work_class(object):
    def __init__(self, shared_data, use_gpu):
        self.x = shared_data['x']

    def process(self, job):
        return sum(self.x) * job

if __name__ == '__main__':
    shared_data = {'x':ones(9000)}
    manager = ClusterManager(work_class, shared_data,
                             machines=['Cyrille-Ulm'],
                             named_pipe=True,
#                             gpu_policy='require_all',
#                             own_max_gpu=0,
#                             own_max_cpu=2,
                             )
    results = manager.process_jobs(arange(manager.total_processes) + 1)
    manager.finished()
    print results



from brian import *
import pycallgraph

dorun = True

def ff(call_stack, module_name, class_name, func_name, full_name):
    if func_name == 'cuba': return True
    if not 'brian' in module_name: return False
    if func_name == '__len__': return False
    if 'units' in module_name: return False
    return True

def ff_nomagicstateupdater(call_stack, module_name, class_name, func_name, full_name):
    if func_name == 'cuba': return True
    if not 'brian' in module_name: return False
    if func_name == '__len__': return False
    if 'units' in module_name: return False
    if func_name == 'magic_state_updater' or func_name == 'prepare':
        return True
    if 'magic' in module_name: return False
    for n in call_stack + [full_name]:
        if 'magic_state_updater' in n:
            return False
        if 'prepare' in n:
            return False
    return True

def ff_magicstateupdater(call_stack, module_name, class_name, func_name, full_name):
    if func_name == 'cuba': return True
    if not 'brian' in module_name: return False
    if func_name == '__len__': return False
    if 'units' in module_name: return False
    if func_name == 'prepare':
        return True
    for n in call_stack + [full_name]:
        if 'prepare' in n:
            return False
    for n in call_stack + [full_name]:
        if 'magic_state_updater' in n:
            return True
    return False

def ff_prepare(call_stack, module_name, class_name, func_name, full_name):
    if func_name == 'cuba': return True
    if not 'brian' in module_name: return False
    if func_name == '__len__': return False
    if 'units' in module_name: return False
    for n in call_stack + [full_name]:
        if 'prepare' in n:
            return True
    return False

def ff_noprepare(call_stack, module_name, class_name, func_name, full_name):
    if func_name == 'cuba': return True
    if not 'brian' in module_name: return False
    if func_name == '__len__': return False
    if 'units' in module_name: return False
    if func_name == 'prepare': return True
    for n in call_stack + [full_name]:
        if 'prepare' in n:
            return False
    return True

def cuba():
    N = 4000
    Ne = int(N * 0.8)
    Ni = N - Ne
    p = 80. / N

    eqs = '''
    dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
    dge/dt = -ge/(5*ms) : volt
    dgi/dt = -gi/(10*ms) : volt
    '''

    pycallgraph.start_trace(filter_func=ff_nomagicstateupdater)
    P = NeuronGroup(N, eqs,
                  threshold= -50 * mV, reset= -60 * mV)
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-NeuronGroup.__init__.no_magic_state_updater.png')

    P.v = -60 * mV + 10 * mV * rand(len(P))
    Pe = P.subgroup(Ne)
    Pi = P.subgroup(Ni)

    pycallgraph.start_trace(filter_func=ff)
    Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=p)
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-Connection.__init__.png')

    Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=p)

    pycallgraph.start_trace(filter_func=ff)
    M = SpikeMonitor(P)
    trace = StateMonitor(P, 'v', record=0)
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-Monitors.__init__.png')

    pycallgraph.start_trace(filter_func=ff_prepare)
    run(.1 * ms)
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-run-prepare.png')

cuba()

def cuba():
    N = 4000
    Ne = int(N * 0.8)
    Ni = N - Ne
    p = 80. / N

    eqs = '''
    dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
    dge/dt = -ge/(5*ms) : volt
    dgi/dt = -gi/(10*ms) : volt
    '''

    pycallgraph.start_trace(filter_func=ff_magicstateupdater)
    P = NeuronGroup(N, eqs,
                  threshold= -50 * mV, reset= -60 * mV)
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-NeuronGroup.__init__.magic_state_updater.png')

    P.v = -60 * mV + 10 * mV * rand(len(P))
    Pe = P.subgroup(Ne)
    Pi = P.subgroup(Ni)

    Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=p)
    Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=p)

    M = SpikeMonitor(P)
    trace = StateMonitor(P, 'v', record=0)

    pycallgraph.start_trace(filter_func=ff_noprepare)
    net = MagicNetwork()
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-run-noprepare-netinit.png')

    pycallgraph.start_trace(filter_func=ff_noprepare)
    net.run(.1 * second)
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-run-noprepare-nonetinit.png')

cuba()

def cuba():
    eqs = '''
    dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
    dge/dt = -ge/(5*ms) : volt
    dgi/dt = -gi/(10*ms) : volt
    '''
    eqs = Equations(eqs)
    pycallgraph.start_trace(filter_func=ff)
    eqs.prepare()
    pycallgraph.stop_trace()
    pycallgraph.make_dot_graph('callgraphs/cuba-bigcallgraph-Equations.prepare.png')

cuba()



from brian import *
set_global_preferences(usenewbrianhears=True)
from brian.hears import *

method = 'function'

#x = whitenoise(1*second)
x = sequence((whitenoise(250*ms), 2.0*whitenoise(750*ms)))

# This class implements the gain (see Filterbank for details)
class GainFilterbank(Filterbank):
    def __init__(self, source, gain=1.0):
        Filterbank.__init__(self, source)
        self.gain = gain
    def buffer_apply(self, input):
        return self.gain*input

if method=='class':
    
    # This is the class for the updater object
    class GainController(object):
        def __init__(self, target, target_rms, time_constant):
            self.target_rms = target_rms
            self.time_constant = time_constant
            self.target = target
        def reinit(self):
            self.sumsquare = 0
            self.numsamples = 0
        def __call__(self, input):
            T = input.shape[0]/self.target.samplerate
            self.sumsquare += sum(input**2)
            self.numsamples += input.size
            rms = sqrt(self.sumsquare/self.numsamples)
            g = self.target.gain
            g_tgt = self.target_rms/rms
            tau = self.time_constant
            self.target.gain = g_tgt+exp(-T/tau)*(g-g_tgt)
    
    updater = GainController(fb, 0.2, 50*ms)

else:
    
    target_rms = 0.2
    tau = 50*ms
    sumsquare = 0
    numsamples = 0
    def updater(input):
        global sumsquare, numsamples
        T = input.shape[0]/fb.samplerate
        sumsquare += sum(input**2)
        numsamples += input.size
        rms = sqrt(sumsquare/numsamples)
        g = fb.gain
        g_tgt = target_rms/rms
        fb.gain = g_tgt+exp(-T/tau)*(g-g_tgt)
    
fb = GainFilterbank(x)

control = ControlFilterbank(fb, x, fb, updater, 10*ms)

#x[:] *= 10.0

y = control.fetch(x.duration)

subplot(211)
plot(x)
plot(y)
subplot(212)
plot(y/x)
show()



from brian import *

def testepsp():
    """Tests whether an alpha function EPSP works algebraically.
    
    The expected behaviour of the network below is that it should solve the
    following differential equation:
    
    taum   dV/dt = -V + x
    taupsp dx/dt = -x + y
    taupsp dy/dt = -y
            V(0) = 0 volt
            x(0) = 0 volt
            y(0) = y0 volt
        
    This gives the following analytical solution for V (computed with Mathematica):
    
    V(t) = (E^(-(t/taum) - t/taupsp)*(-(E^(t/taum)*t*taum) + 
       E^(t/taum)*t*taupsp - E^(t/taum)*taum*taupsp + 
       E^(t/taupsp)*taum*taupsp)*y0)/(taum - taupsp)^2
    
    This doesn't have an analytical solution for the maximum value of V, but the
    following numerical value was computed with the analytic formula:
    
            Vmax = 0.136889 mvolt  (accurate to that many sig figs)
    at time    t = 1.69735 ms (accurate to +/- 0.00001ms)
    
    The Brian network consists of two neurons, one governed by the differential
    equations given above, the other fires a single spike at time t=0 and is
    connected to the first
    """
    reinit_default_clock()
    clock = Clock(dt=0.1 * ms)
    expected_vmax = 0.136889 * mvolt
    expected_vmaxtime = 1.69735 * msecond
    desired_vmaxaccuracy = 0.001 * mvolt
    desired_vmaxtimeaccuracy = max(clock.dt, 0.00001 * ms)
    taum = 10 * ms
    taupsp = 0.325 * ms
    y0 = 4.86 * mV
    P = NeuronGroup(N=1, model='''
                  dV/dt = (-V+x)*(1./taum) : volt
                  dx/dt = (-x+y)*(1./taupsp) : volt
                  dy/dt = -y*(1./taupsp) : volt
                  ''',
                  threshold=100 * mV, reset=0 * mV)
    Pinit = SpikeGeneratorGroup(1, [(0, 0 * ms)])
    C = Connection(Pinit, P, 'y')
    C.connect_full(Pinit, P, y0)
    M = StateMonitor(P, 'V', record=0)
    run(10 * ms)
    V = M[0]
    Vmax = 0
    Vi = 0
    for i in range(len(V)):
        if V[i] > Vmax:
            Vmax = V[i]
            Vi = i
    Vmaxtime = M.times[Vi] * second
    Vmax = Vmax * volt
    assert abs(Vmax - expected_vmax) < desired_vmaxaccuracy
    assert abs(Vmaxtime - expected_vmaxtime) < desired_vmaxtimeaccuracy

if __name__ == '__main__':
    testepsp()



import pycuda.autoinit as autoinit
import pycuda.driver as drv
from pycuda.gpuarray import GPUArray
from pycuda import gpuarray
import bisect
import numpy, pylab, time
from itertools import repeat
import cProfile as profile
import pstats
import brian

N = 512 * 256
blocksize = 512
duration = 10000
record = False
doprofile = False
sparseness = 80. / N

if drv.get_version() == (2, 0, 0): # cuda version
    precision = 'float'
elif drv.get_version() > (2, 0, 0):
    precision = 'double'
else:
    raise Exception, "CUDA 2.0 required"

if precision == 'double':
    mydtype = numpy.float64
else:
    mydtype = numpy.float32

block = (blocksize, 1, 1)
grid = (N / blocksize, 1)
Ne = int(N * 0.8)
Ni = N - Ne

mod = drv.SourceModule("""
__global__ void stateupdate(SCALAR *V, SCALAR *ge, SCALAR *gi)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x; 
    SCALAR V__tmp = (ge[i]+gi[i]-(V[i]+0.049))/0.02;
    SCALAR ge__tmp = -ge[i]/0.005;
    SCALAR gi__tmp = -gi[i]/0.01;
    V[i] += 0.0001*V__tmp;
    ge[i] += 0.0001*ge__tmp;
    gi[i] += 0.0001*gi__tmp;
}

__global__ void threshold(SCALAR *V, int *spikes, bool *spiked, unsigned int *global_j, int N)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    bool this_spiked = V[i]>-0.05; 
    spiked[i] = this_spiked;
    if(this_spiked) // && i<N) // can leave out i<N check if N%blocksize=0
    {
        unsigned int j = atomicInc(global_j, N);
        spikes[j] = i;
    }
}

__global__ void single_thread_threshold(SCALAR *V, int *spikes, bool *spiked, unsigned int *global_j, int N)
{
    unsigned int j = 0;
    for(int i=0;i<N;i++)
    {
        if(V[i]>-0.05)
        {
            spiked[i] = true;
            spikes[j++] = i;
        } else
        {
            spiked[i] = false;
        }
    }
    *global_j = j;
}

__global__ void propagate(int *spikes, int numspikes, SCALAR *v, SCALAR *W, int N)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    for(int j=0; j<numspikes; j++)
        v[i] += W[i+N*spikes[j]];
}

__global__ void propagate_spike(int spike, SCALAR *alldata, int *allj, int *rowind, SCALAR *V)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    int startindex = rowind[spike];
    if(i>=rowind[spike+1]-startindex) return;
    i += startindex;
    V[allj[i]] += alldata[i];
}

__global__ void reset(SCALAR *V, bool *spiked)
{
    int i = blockIdx.x * blockDim.x + threadIdx.x;
    V[i] = (V[i]*!spiked[i])+(-0.06)*spiked[i]; // i.e. V[i]=-0.06 if spiked[i]
}
""".replace('SCALAR', precision))
stateupdate = mod.get_function("stateupdate")
threshold = mod.get_function("threshold")
single_thread_threshold = mod.get_function("single_thread_threshold")
propagate = mod.get_function("propagate")
propagate_spike = mod.get_function("propagate_spike")
reset = mod.get_function("reset")

V = gpuarray.to_gpu(numpy.array(numpy.random.rand(N) * 0.01 - 0.06, dtype=mydtype))
ge = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))
gi = gpuarray.to_gpu(numpy.zeros(N, dtype=mydtype))

we = 0.00162
wi = -0.009
P = brian.NeuronGroup(N, model='V:1')
Pe = P.subgroup(Ne)
Pi = P.subgroup(Ni)
Ce = brian.Connection(Pe, P, sparseness=sparseness, weight=we).W.connection_matrix()
Ci = brian.Connection(Pi, P, sparseness=sparseness, weight=wi).W.connection_matrix()
Ce.alldata = numpy.array(Ce.alldata, dtype=mydtype)
Ci.alldata = numpy.array(Ci.alldata, dtype=mydtype)
Ce_alldata = drv.mem_alloc(Ce.alldata.nbytes)
drv.memcpy_htod(Ce_alldata, Ce.alldata)
Ce_allj = drv.mem_alloc(Ce.allj.nbytes)
drv.memcpy_htod(Ce_allj, Ce.allj)
Ce_rowind = drv.mem_alloc(Ce.rowind.nbytes)
drv.memcpy_htod(Ce_rowind, Ce.rowind)
Ci_alldata = drv.mem_alloc(Ci.alldata.nbytes)
drv.memcpy_htod(Ci_alldata, Ci.alldata)
Ci_allj = drv.mem_alloc(Ci.allj.nbytes)
drv.memcpy_htod(Ci_allj, Ci.allj)
Ci_rowind = drv.mem_alloc(Ci.rowind.nbytes)
drv.memcpy_htod(Ci_rowind, Ci.rowind)

gpu_spikes = drv.mem_alloc(4 * N)
gpu_spikes_exc = drv.mem_alloc(4 * N)
gpu_spikes_inh = drv.mem_alloc(4 * N)
gpu_spiked = gpuarray.to_gpu(numpy.zeros(N, dtype=bool))
gpu_spike_index = drv.mem_alloc(4)
master_spikes = numpy.zeros(N, dtype=int)
spike_index = numpy.zeros(1, dtype=numpy.uint32)
drv.memcpy_htod(gpu_spikes, master_spikes)
drv.memcpy_htod(gpu_spike_index, spike_index)

recspikes = []
recv0 = numpy.zeros(duration, dtype=mydtype)
totalnspikes = 0

stateupdate.prepare(('i', 'i', 'i'), block)
threshold.prepare(('i', 'i', 'i', 'i', numpy.int32), block)
single_thread_threshold.prepare(('i', 'i', 'i', 'i', numpy.int32), (1, 1, 1))
propagate.prepare(('i', numpy.int32, 'i', 'i', numpy.int32), block)
propagate_spike.prepare((numpy.int32, 'i', 'i', 'i', 'i'), block)
reset.prepare(('i', 'i'), block)

stateupdate_args = (grid, int(V.gpudata), int(ge.gpudata), int(gi.gpudata))
threshold_args = (grid, int(V.gpudata), int(gpu_spikes), int(gpu_spiked.gpudata), int(gpu_spike_index), numpy.int32(N))
single_thread_threshold_args = ((1, 1), int(V.gpudata), int(gpu_spikes), int(gpu_spiked.gpudata), int(gpu_spike_index), numpy.int32(N))
reset_args = (grid, int(V.gpudata), int(gpu_spiked.gpudata))
propagate_spike_end_args_exc = (int(Ce_alldata), int(Ce_allj), int(Ce_rowind), int(ge.gpudata))
propagate_spike_end_args_inh = (int(Ci_alldata), int(Ci_allj), int(Ci_rowind), int(gi.gpudata))

def run_sim():
    global totalnspikes, recspikes, recv0
    for t in xrange(duration):
        #stateupdate(V, ge, gi, block=block, grid=grid)
        if t == 0:
            stateupdate.prepared_call(*stateupdate_args)
        else:
            stateupdate.launch_grid(*grid)
        spike_index[0] = 0
        drv.memcpy_htod(gpu_spike_index, spike_index)
        #threshold(V, gpu_spikes, gpu_spiked, gpu_spike_index, numpy.int32(N), block=block, grid=grid)
        if t == 0:
            threshold.prepared_call(*threshold_args)
        else:
            threshold.launch_grid(*grid)
        # MUCH! slower
        #single_thread_threshold.prepared_call(*single_thread_threshold_args)
        drv.memcpy_dtoh(spike_index, gpu_spike_index)
        numspikes = spike_index[0]
        spikes = master_spikes[:numspikes]
        drv.memcpy_dtoh(spikes, gpu_spikes)
        spikes.sort()
        numspikes_exc = bisect.bisect_left(spikes, Ne)
        numspikes_inh = numspikes - numspikes_exc
        #drv.memcpy_htod(gpu_spikes_exc, spikes[:numspikes_exc])
        #drv.memcpy_htod(gpu_spikes_inh, spikes[numspikes_exc:]-Ne)
        #propagate(gpu_spikes_exc, numpy.int32(numspikes_exc), ge, CeW, numpy.int32(N), block=block, grid=grid)
        #propagate.prepared_call(grid, int(gpu_spikes_exc), numpy.int32(numspikes_exc), int(ge.gpudata), int(CeW.gpudata), numpy.int32(N))
        #propagate(gpu_spikes_inh, numpy.int32(numspikes_inh), gi, CiW, numpy.int32(N), block=block, grid=grid)
        #propagate.prepared_call(grid, int(gpu_spikes_inh), numpy.int32(numspikes_inh), int(gi.gpudata), int(CiW.gpudata), numpy.int32(N))
        for i in spikes[:numspikes_exc]:
            ncols = len(Ce.rowj[i])
            if ncols % 512 == 0:
                spikegrid = (ncols / 512, 1)
            else:
                spikegrid = (ncols / 512 + 1, 1)
            propagate_spike.prepared_call(spikegrid, numpy.int32(i), *propagate_spike_end_args_exc)
        for i in spikes[numspikes_exc:] - Ne:
            ncols = len(Ci.rowj[i])
            if ncols % 512 == 0:
                spikegrid = (ncols / 512, 1)
            else:
                spikegrid = (ncols / 512 + 1, 1)
            propagate_spike.prepared_call(spikegrid, numpy.int32(i), *propagate_spike_end_args_inh)
        #reset(V, gpu_spiked, block=block, grid=grid)
        if t == 0:
            reset.prepared_call(*reset_args)
        else:
            reset.launch_grid(*grid)
        if record:
            recspikes += zip(spikes, repeat(t))
            drv.memcpy_dtoh(recv0[t:t + 1], V.gpudata)
        totalnspikes += numspikes

if doprofile:
    start = time.time()
    profile.run('run_sim()', 'cudacuba.prof')
    end = time.time()
    stats = pstats.Stats('cudacuba.prof')
    #stats.strip_dirs()
    stats.sort_stats('cumulative', 'calls')
    stats.print_stats(50)
else:
    start = time.time()
    run_sim()
    end = time.time()
print 'GPU time:', end - start
print 'Number of spikes:', totalnspikes

if record:
    i, t = zip(*recspikes)
    pylab.subplot(121)
    pylab.plot(t, i, '.')
    pylab.subplot(122)
    pylab.plot(recv0)
    pylab.show()



'''
Integrate-and-fire neurons with noise
'''
from brian import *

tau = 10 * ms
sigma = .5
N = 100
J = -1
mu = 2

eqs = """
dv/dt=mu/tau+sigma/tau**.5*xi : 1
"""

group = NeuronGroup(N, model=eqs, threshold=1, reset=0)

C = Connection(group, group, 'v')
for i in range(N):
    C[i, (i + 1) % N] = J

#C.connect_full(group,group,weight=J)
#for i in range(N):
#    C[i,i]=0

S = SpikeMonitor(group)
trace = StateMonitor(group, 'v', record=True)

run(500 * ms)
i, t = S.spikes[-1]

subplot(211)
raster_plot(S)
subplot(212)
plot(trace.times / ms, trace[0])
show()



'''
Tool to set the current version of Brian in the various places that have it,
i.e.:

* Global __version__ in __init__.py
* setup.py version
* docs version
* README.txt version
'''

import os, sys, brian, re
from update_release_date import setreleasedate

def setversion(version):
    docs_release = version
    major = docs_release[:docs_release.find('.')]
    minor = docs_release[docs_release.find('.') + 1:docs_release.find('.', docs_release.find('.') + 1)]
    docs_version = major + '.' + minor
    pathname, filename = os.path.split(__file__)
    os.chdir(pathname)
    os.chdir('../../../')
    # update setup.py
    setup_py = open('setup.py', 'r').read()
    setup_py = re.sub("version\s*=\s*'.*?'", "version = '" + version + "'", setup_py)
    open('setup.py', 'w').write(setup_py)
    # update __init__.py
    init_py = open('brian/__init__.py', 'r').read()
    init_py = re.sub("__version__\s*=\s*'.*?'", "__version__ = '" + version + "'", init_py)
    open('brian/__init__.py', 'w').write(init_py)
    # update sphinx docs
    conf_py = open('docs_sphinx/conf.py', 'r').read()
    conf_py = re.sub("version\s*=\s*'.*?'", "version = '" + docs_version + "'", conf_py)
    conf_py = re.sub("release\s*=\s*'.*?'", "release = '" + docs_release + "'", conf_py)
    open('docs_sphinx/conf.py', 'w').write(conf_py)
    # update README.txt
    readme_txt = open('README.txt', 'r').read()
    readme_txt = re.sub(r"Version: .*\n", "Version: " + version + '\n', readme_txt)
    open('README.txt', 'w').write(readme_txt)

def user_input_setversion():
    print 'Current version of Brian is', brian.__version__
    version = raw_input('Enter new Brian version number: ')
    version = version.strip()
    print 'Changing to new version', version
    setversion(version)
    print 'Done'

if __name__ == '__main__':
    print 'Current version of Brian is', brian.__version__
    if len(sys.argv) < 2:
        version = raw_input('Enter new Brian version number: ')
    else:
        version = sys.argv[1]
    version = version.strip()
    print 'Changing to new version', version
    setversion(version)
    setreleasedate()
    print 'Done'



from brian import *

eqs = '''
dv/dt = (ge+gi-(v+49*mV))/(20*ms) : volt
dge/dt = -ge/(5*ms) : volt
dgi/dt = -gi/(10*ms) : volt
'''
P = NeuronGroup(4000, model=eqs,
              threshold= -50 * mV, reset= -60 * mV)
P.v = -60 * mV + 10 * mV * rand(len(P))
Pe = P.subgroup(3200)
Pi = P.subgroup(800)
Ce = Connection(Pe, P, 'ge', weight=1.62 * mV, sparseness=0.02)
Ci = Connection(Pi, P, 'gi', weight= -9 * mV, sparseness=0.02)
M = SpikeMonitor(P)
@network_operation(clock=EventClock(dt=1 * second))
def clearspikes():
    M.reinit()
    print 'Simulated', defaultclock.t

srv = RemoteControlServer()

run(1e10 * second)

print 'Finished!'



'''
Example of the use of the class :class:`~brian.hears.Butterworth` available in
the library. In this example, a white noise is filtered by a bank of butterworth
bandpass filters and lowpass filters which are different for every channels. The
centre or cutoff frequency of the filters are linearly taken between 100kHz and
1000kHz and its bandwidth frequency increases linearly with frequency.
'''
from brian import *
from brian.hears import *

level = 50*dB  # level of the input sound in rms dB SPL
sound = whitenoise(100*ms).ramp()
sound = sound.atlevel(level)
order = 2 #order of the filters

#### example of a bank of bandpass filter ################
nchannels = 50
center_frequencies = linspace(100*Hz, 1000*Hz, nchannels) 
bw = linspace(50*Hz, 300*Hz, nchannels) # bandwidth of the filters
#arrays of shape (2 x nchannels) defining the passband frequencies (Hz)
fc = vstack((center_frequencies-bw/2, center_frequencies+bw/2))

filterbank = Butterworth(sound, nchannels, order, fc, 'bandpass')

filterbank_mon = filterbank.process()

figure()
subplot(211)
imshow(flipud(filterbank_mon.T), aspect='auto')    

### example of a bank of lowpass filter ################
nchannels = 50
cutoff_frequencies = linspace(200*Hz, 1000*Hz, nchannels) 

filterbank = Butterworth(sound, nchannels, order, cutoff_frequencies, 'low')

filterbank_mon = filterbank.process()

subplot(212)
imshow(flipud(filterbank_mon.T), aspect='auto')    
show()



__all__ = ['euler_scheme', 'rk2_scheme', 'exp_euler_scheme']

euler_scheme = [
    (('foreachvar', 'nonzero'),
        '''
        $vartype ${var}__tmp = $var_expr
        '''),
    (('foreachvar', 'nonzero'),
        '''
        $var += ${var}__tmp*dt
        ''')
    ]

rk2_scheme = [
    (('foreachvar', 'all'),
        '''
        $vartype ${var}__buf = $var_expr
        $vartype ${var}__half = (.5*dt)*${var}__buf
        ${var}__half += $var
        '''),
    (('foreachvar', 'nonzero'),
        '''
        ${var}__buf = @substitute(var_expr, dict((var, var+'__half') for var in vars))
        $var += dt*${var}__buf
        ''')
    ]

exp_euler_scheme = [
    (('foreachvar', 'nonzero'),
        '''
        $vartype ${var}__B = @substitute(var_expr, {var:0})
        $vartype ${var}__A = @substitute(var_expr, {var:1})
        ${var}__A -= ${var}__B
        ${var}__B /= ${var}__A
        ${var}__A *= dt
        '''),
    (('foreachvar', 'nonzero'),
        '''
        $var += ${var}__B
        $var *= exp(${var}__A)
        $var -= ${var}__B
        ''')
    ]



'''
Ideas for weight monitor:

Should have following features:

* Record from a subset of weights
* Convert to Brian or scipy sparse matrix format
* Callback feature
* Record to file
* Display in GUI
'''
from brian import *
from random import sample
import matplotlib.cm as cm
try:
    import pygame
except ImportError:
    pygame = None

__all__ = ['ConnectionMonitor']

class ConnectionMonitor(NetworkOperation):
    '''
    Monitors synaptic connection weights
    
    Initialisation arguments:
    
    ``C``
        The connection from which to monitor weight values.
    ``matrix='W'``
        Which matrix to record from, 'W' is weights but you can also record
        delays with ``matrix='delay'``.
    ``synapses=None``
        Specify which synapses to record. Can take the following values:
        
        An integer
            Will record this many randomly selected synapses (or less if
            there are less).
        A value between 0 and 1
            Will record this fraction of the synapses, chosen at random.
        ``True``, ``None``
            Records all the synapses
        A list
            The list should consist of pairs ``(i, j)`` to record those
            synapses only.
            
        Note that if an integer or fraction is specified, for a sparse matrix,
        the set of synapses that will be recorded will be chosen at random
        from the nonzero values at the beginning of the run. If you are using
        a :class:`DynamicConnectionMatrix` you might want to specify the
        synapses to record from explicitly.
    ``callback=None``
        A callback function, each time a matrix ``M`` is recorded,
        ``callback(M)`` will be called. This could be used for saving matrices
        to a file, for example, or for runtime analysis. See below for recorded
        matrix format.
    ``store=False``
        Whether or not to keep a copy of the recorded matrices in memory,
        specifically in the ``values`` attribute (see below).
    ``display=False``
        Whether or not to display images at runtime. Can take any of the
        following values:
        
        ``'pygame'``
            Displays the matrix using the pygame module, which must be installed
            if you want to use this. Only one matrix can be recorded at a time
            using pygame.
        
        Note that if ``display`` is not ``False`` it will replace the image
        callback provided (if any).
    ``image_callback=None``
        Each time an image ``I`` is recorded, ``image_callback(I)`` is called.
        See below for image format.
    ``size=None``
        Specifies the dimension of the image. A fairly crude scaling will be used
        if the dimensions of the image are different to those of the matrix.
    ``cmap=jet``
        The colourmap to use. Various values are available in
        ``matplotlib.cm``.
    ``wmin=None, wmax=None``
        The minimum and maximum weights for use with the colourmap. If not
        specified, for each recorded matrix the minimum and maximum values will
        be used.
    ``clock=None``
        By default, matrices will be recorded every 1s of simulated time.
    ``when='end'``
        When the matrices are recorded (see :class:`NetworkOperation`).
        
    **Matrix format**
    
    Matrices are recorded in one of two formats, either a numpy ``ndarray``
    if the matrix to be recorded is dense and all synapses are being recorded,
    or a Brian :class:`SparseMatrix` (which is derived from
    ``scipy.sparse.lil_matrix``) in all other cases. If a subset of synapses
    to record has been selected, the sparse matrix will consist only of those
    synapses.
    
    **Image format**
    
    The images are numpy arrays with shape ``(width, height, 4)`` and 
    ``dtype=uint8``. The last axis is the colour in the format RGBA, with
    values between 0 and 255. The A component is alpha, and will always be
    255.
        
    **Attributes**
    
    ``values``
        If ``store=True`` this consists of a list of pairs ``(t, M)`` giving
        the recorded matrix ``M`` at time ``t``.
    '''
    def __init__(self, C, matrix='W', synapses=None,
                 callback=None, store=False,
                 display=False,
                 image_callback=None, size=None,
                 cmap=cm.jet, wmin=None, wmax=None,
                 clock=None, when='end',
                 ):
        self.C = C
        self.matrix = matrix
        self.synapses = synapses
        self.synapse_set = None
        self.callback = callback
        self.display = display
        self.store = store
        self.image_callback = image_callback
        Wh, Ww = getattr(C, matrix).shape
        if size is None:
            size = Ww, Wh
        w, h = size
        self.size = (h, w)
        self.wmin = wmin
        self.wmax = wmax
        self.cmap = cmap
        if clock is None:
            clock = EventClock(dt=1*second)
        if display=='pygame':
            if pygame is None:
                warnings.warn('Need pygame module to use pygame display mode.')
            else:
                self.screen = pygame.display.set_mode(self.size[::-1])
                self.screen_arr = pygame.surfarray.pixels3d(self.screen)
                self.image_callback = self.pygame_image_callback            
        NetworkOperation.__init__(self, lambda:None, clock=clock, when=when)
        self.reinit()

    def reinit(self):
        self.values = []
        
    def get_matrix(self):
        W = getattr(self.C, self.matrix)
        sparseW = isinstance(W.get_row(0), SparseConnectionVector)
        if not(self.synapses is None or self.synapses is True):
            if self.synapse_set is None:
                # The first time we run we generate the synapse set
                # We don't do this on initialisation because the user
                # might initialise the matrix values before defining the
                # monitor.
                if sparseW:
                    nnz = W.getnnz()
                else:
                    nnz = W.shape[0]*W.shape[1]
                synapses = self.synapses
                if isinstance(synapses, float):
                    synapses = int(synapses*nnz)
                if isinstance(synapses, int):
                    if synapses>=nnz:
                        # recording all synapses, so use the code below
                        self.synapses = None
                    else:
                        # We want to pick a sample of indices at random, so
                        # we pick a subset of [0...nnz-1] and pick out the
                        # corresponding synapses (i,j) if they are arranged
                        # in order
                        indices = sample(xrange(nnz), synapses)
                        indices.sort()
                        indices = array(indices, dtype=int)
                        if not sparseW:
                            # For dense matrices there is a simple conversion
                            # from 1D indices to 2D indices
                            i = indices%W.shape[0]
                            j = indices/W.shape[0]
                            synapses = zip(i, j)
                        else:
                            # For sparse matrices, we need to go through row
                            # by row and pick out the corresponding indices
                            # Note that we don't return a list of pairs
                            # (i,j) in this case, but directly create the
                            # synapse_set as it is more efficient
                            ss = SparseMatrix(W.shape)
                            for i in xrange(W.shape[0]):
                                row = W.get_row(i)
                                I = indices<len(row)
                                ss[i, row.ind[indices[I]]] = 1
                                indices = indices[-I]-len(row)
                            self.synapse_set = ss
                # synapses should now be a list of pairs (i, j)
                if self.synapse_set is None:
                    # synapse_set is a sparse matrix with nonzero elements
                    # in the places we want to record from
                    self.synapse_set = SparseMatrix(W.shape)
                    for u, v in synapses:
                        self.synapse_set[u, v] = 1
            # now we record a new sparse matrix by looping through the rows
            # and copying the data across
            M = SparseMatrix(W.shape)
            for i in xrange(W.shape[0]):
                inds = self.synapse_set.rows[i]
                if len(inds):
                    row = W.get_row(i)
                    # we convert to dense vector because we want to extract
                    # certain indices, and these indices may not exist in
                    # the underlying sparse vector (i.e. they may be zeros)
                    if isinstance(row, SparseConnectionVector):
                        row = row.todense()
                    M[i, inds] = row[inds]
        if self.synapses is None or self.synapses is True:
            # If we're recording everything it's fairly simple, we just
            # make a copy.
            if sparseW:
                M = SparseMatrix(W.shape)
            else:
                M = zeros(W.shape)
            for i in xrange(W.shape[0]):
                row = W.get_row(i)
                if sparseW:
                    M[i, row.ind] = asarray(row)
                else:
                    M[i, :] = row
        return M
    
    def get_image(self, W=None):
        '''
        Returns an image corresponding to the matrix W, scaling down to
        the appropriate size.
        '''
        wmin, wmax = self.wmin, self.wmax
        if W is None:
            W = getattr(self.C, self.matrix)
        h, w = self.size
        Wh, Ww = W.shape
        # We initially generate the image at the maximum size of the matrix,
        # and scale up if the requested size was bigger
        if w>Ww: w = Ww
        if h>Wh: h = Wh
        image = zeros((w, h))
        # We're going to do some crude scaling by adding all the weights and
        # dividing by the number of itmes, for each pixel of the image
        image_numitems = zeros((w, h), dtype=int)
        ind = arange(W.shape[1])
        # We will do histograms for each row, adding up the corresponding
        # weights for indices in these bins
        bins = hstack(((arange(w)*Ww)/w, Ww))
        for i in xrange(W.shape[0]):
            u = h-1-((i*h)//Wh) # target coordinate, scaled
            row = W.get_row(i)
            if isinstance(row, SparseConnectionVector):
                ind = row.ind
            numitems, _ = histogram(ind, bins)
            vals, _ = histogram(ind, bins, weights=row)
            image[:, u] += vals
            image_numitems[:, u] += numitems
        image_numitems[image_numitems==0] = 1
        image = image/image_numitems
        if wmin is None: wmin = amin(image)
        if wmax is None: wmax = amax(image)
        if wmax - wmin < 1e-20: wmax = wmin + 1e-20
        image = self.cmap(clip((image - wmin) / (wmax - wmin), 0, 1), bytes=True)
        # Scale up if the requested image size is larger than the matrix
        ih, iw = self.size
        if ih>h or iw>w:
            yind = (arange(ih)*h)/ih
            xind = (arange(iw)*w)/iw
            xind, yind = meshgrid(xind, yind)
            image = image[xind.T, yind.T, :]
        return image
        
    def pygame_image_callback(self, image):
        self.screen_arr[:, :, :3] = image[:, :, :3]
        pygame.display.flip()
        pygame.event.pump()
        
    def __call__(self):
        W = self.get_matrix()
        if self.callback is not None:
            self.callback(W)
        if self.store:
            self.values.append((self.clock.t, W))
        if self.image_callback is not None:
            self.image_callback(self.get_image())

if __name__=='__main__':
    if 0:
        N = 1000
        taum = 10 * ms
        tau_pre = 20 * ms
        tau_post = tau_pre
        Ee = 0 * mV
        vt = -54 * mV
        vr = -60 * mV
        El = -74 * mV
        taue = 5 * ms
        F = 15 * Hz
        gmax = .01
        dA_pre = .01
        dA_post = -dA_pre * tau_pre / tau_post * 1.05
        
        eqs_neurons = '''
        dv/dt=(ge*(Ee-vr)+El-v)/taum : volt   # the synaptic current is linearized
        dge/dt=-ge/taue : 1
        '''
        
        input = PoissonGroup(N, rates=F)
        neurons = NeuronGroup(1, model=eqs_neurons, threshold=vt, reset=vr)
        synapses = Connection(input, neurons, 'ge', weight=rand(len(input), len(neurons)) * gmax)
        neurons.v = vr
        
        #stdp=ExponentialSTDP(synapses,tau_pre,tau_post,dA_pre,dA_post,wmax=gmax)
        ## Explicit STDP rule
        eqs_stdp = '''
        dA_pre/dt=-A_pre/tau_pre : 1
        dA_post/dt=-A_post/tau_post : 1
        '''
        dA_post *= gmax
        dA_pre *= gmax
        stdp = STDP(synapses, eqs=eqs_stdp, pre='A_pre+=dA_pre;w+=A_post',
                  post='A_post+=dA_post;w+=A_pre', wmax=gmax)
        
        rate = PopulationRateMonitor(neurons)
        
        M = ConnectionMonitor(synapses, store=True, synapses=200)
        
        run(100 * second, report='text')
        
        Z = zeros((100, 1000))
        
        for i, (_, m) in enumerate(M.values):
            #print m[:, 0].__class__
            Z[i, :] = m.todense().flatten()
            
        imshow(Z, aspect='auto')
        show()
    else:
        Nin = 200
        Nout = 200
        duration = 40 * second
    
        Fmax = 50 * Hz
    
        tau = 10 * ms
        taue = 2 * ms
        taui = 5 * ms
        sigma = 0.#0.4
        # Note that factor (tau/taue) makes integral(v(t)) the same when the connection
        # acts on ge as if it acted directly on v.
        eqs = Equations('''
        #dv/dt = -v/tau + sigma*xi/(2*tau)**.5 : 1
        dv/dt = (-v+(tau/taue)*ge-(tau/taui)*gi)/tau + sigma*xi/(2*tau)**.5 : 1
        dge/dt = -ge/taue : 1
        dgi/dt = -gi/taui : 1
        excitatory = ge
        inhibitory = gi
        ''')
        reset = 0
        threshold = 1
        refractory = 0 * ms
        taup = 5 * ms
        taud = 5 * ms
        Ap = .1
        Ad = -Ap * taup / taud * 1.2
        wmax_ff = 0.1
        wmax_rec = wmax_ff
        wmax_inh = wmax_rec
    
        width = 0.2
    
        recwidth = 0.2
    
        Gin = PoissonGroup(Nin)
        Gout = NeuronGroup(Nout, eqs, reset=reset, threshold=threshold, refractory=refractory)
        ff = Connection(Gin, Gout, 'excitatory', structure='dense')
        for i in xrange(Nin):
            ff[i, :] = (rand(Nout) > .5) * wmax_ff
        rec = Connection(Gout, Gout, 'excitatory')
        for i in xrange(Nout):
            d = abs(float(i) / Nout - linspace(0, 1, Nout))
            d[d > .5] = 1. - d[d > .5]
            dsquared = d ** 2
            prob = exp(-dsquared / (2 * recwidth ** 2))
            prob[i] = -1
            inds = (rand(Nout) < prob).nonzero()[0]
            w = rand(len(inds)) * wmax_rec
            rec[i, inds] = w
    
        inh = Connection(Gout, Gout, 'inhibitory', sparseness=1, weight=wmax_inh)
    
        stdp_ff = ExponentialSTDP(ff, taup, taud, Ap, Ad, wmax=wmax_ff)
        stdp_rec = ExponentialSTDP(rec, taup, taud, Ap, Ad, wmax=wmax_rec)
    
        M = ConnectionMonitor(ff, display='pygame',
                              #size=(500,500),
                              #synapses=10000,
                              )
        
        run(0 * ms)
        @network_operation(clock=EventClock(dt=20 * ms))
        def stimulation():
            Gin.rate = Fmax * exp(-(linspace(0, 1, Nin) - rand())**2 / (2 * width ** 2))
    
        run(40*second, report='stderr')

        if 0:
            for i, (_, m) in enumerate(M.values):
                if i%16==0:
                    figure()
                subplot(4, 4, i%16+1)
                imshow(array(m.todense()), aspect='auto',
                       #interpolation='nearest'
                       )
            show()
        


from brian import Equations, NeuronGroup, Clock, CoincidenceCounter, Network, zeros, array, \
                    ones, kron, ms, second, concatenate, hstack, sort, nonzero, diff, TimedArray, \
                    reshape, sum, log, Monitor, NetworkOperation, defaultclock, linspace, vstack, \
                    arange, sort_spikes, rint, SpikeMonitor, Connection, int32, double
from brian.tools.statistics import firing_rate, get_gamma_factor

try:
    import pycuda
    import pycuda.driver as drv
    import pycuda
    from pycuda import gpuarray
    can_use_gpu = True
except ImportError:
    can_use_gpu = False
from brian.experimental.codegen.integration_schemes import *
import sys, cPickle




class Criterion(Monitor, NetworkOperation):
    """
    Abstract class from which modelfitting criterions should derive.
    Derived classes should implement the following methods:
    
    ``initialize(self, **params)``
        Called once before the simulation. ```params`` is a dictionary with criterion-specific
        parameters. 
    
    ``timestep_call(self)``
        Called at every timestep.
    
    ``spike_call(self, neurons)``
        Called at every spike, with the spiking neurons as argument.
    
    ``get_values(self)``
        Called at the end, returns the criterion values.
        
    You have access to the following methods:
    
    ``self.get_value(self, varname)``
        Returns the value of the specified variable for all neurons (vector).
    
    You have access to the following attributes:
    
    ``self.step``
        The time step (integer)
    
    ``self.group``
        The NeuronGroup
    
    ``self.traces=None``
        Target traces.
        A 2-dimensional K*T array where K is the number of targets, and T the total number of timesteps.
        It is still a 2-dimensional array when K=1. It is None if not specified.
        
    ``self.spikes=None``
        A list of target spike trains : [(i,t)..] where i is the target index and t the spike time.
    
    ``self.N``
        The number of neurons in the NeuronGroup
    
    ``self.K=1``
        The number of targets.
    
    ``self.duration``
        The total duration of the simulation, in seconds.
    
    ``self.total_steps``
        The total number of time steps.
    
    ``self.dt``
        The timestep duration, in seconds.
    
    ``self.delays=zeros(self.n)``
        The delays for every neuron, in seconds. The delay is relative to the target.
    
    ``self.onset=0*ms``
        The onset, in seconds. The first timesteps, before onset, should be discarded in the criterion.
    
    ``self.intdelays=0``
        The delays, but in number of timesteps (``int(delays/dt)``).
    
    NOTE: traces and spikes have all been sliced before being passed to the criterion object
    """
    def __init__(self, group, traces=None, spikes=None, targets_count=1, duration=None, onset=0*ms, 
                 spikes_inline=None, spikes_offset=None,
                 traces_inline=None, traces_offset=None,
                 delays=None, when='end', **params):
        NetworkOperation.__init__(self, None, clock=group.clock, when=when)
        self.group = group
        
        # needed by SpikeMonitor
        self.source = group
        self.source.set_max_delay(0)
        self.delay = 0
        
        self.N = len(group) # number of neurons
        self.K = targets_count # number of targets
        self.dt = self.clock.dt
        if traces is not None:
            self.traces = array(traces) # KxT array
            if self.traces.ndim == 1:
                self.traces = self.traces.reshape((1,-1))
            assert targets_count==self.traces.shape[0]
        self.spikes = spikes
        # get the duration from the traces if duration is not specified in the constructor
        if duration is None: duration = self.traces.shape[1] # total number of steps
        self.duration = duration
        self.total_steps = int(duration/self.dt)
        if delays is None: delays = zeros(self.n)
        self.delays = delays
        self.onset = int(onset/self.dt)
        self.intdelays = array(self.delays/self.clock.dt, dtype=int)
        self.mindelay = min(delays)
        self.maxdelay = max(delays)
        # the following data is sliced
        self.spikes_inline = spikes_inline
        self.spikes_offset = spikes_offset
        self.traces_inline = traces_inline
        self.traces_offset = traces_offset
        if self.spikes is not None:
            # target spike count and rates
            self.target_spikes_count = self.get_spikes_count(self.spikes)
            self.target_spikes_rate = self.get_spikes_rate(self.spikes)
        self.initialize(**params)

    def step(self):
        """
        Return the current time step
        """
        return int(round(self.clock.t/self.dt))
    
    def get_spikes_count(self, spikes):
        count = zeros(self.K)
        for (i,t) in spikes:
            count[i] += 1
        return count
    
    def get_spikes_rate(self, spikes):
        count = self.get_spikes_count(spikes)
        return count*1.0/self.duration
    
    def get_gpu_code(self): # TO IMPLEMENT
        """
        Returns CUDA code snippets to put at various places in the CUDA code template.
        It must return a dictionary with the following items:
            %CRITERION_DECLARATION%: kernel declaration code
            %CRITERION_INIT%: initialization code
            %CRITERION_TIMESTEP%: main code, called at every time step
            %CRITERION_END%: finalization code
        """
        log_warn("GPU code not implemented by the derived criterion class")
    
    def initialize_cuda_variables(self): # TO IMPLEMENT
        """
        Initialize criterion-specific CUDA variables here.
        """
        pass
    
    def get_kernel_arguments(self): # TO IMPLEMENT
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        pass
    
    def initialize(self): # TO IMPLEMENT
        """
        Override this method to initialize the criterion before the simulation
        """
        pass
    
    def timestep_call(self): # TO IMPLEMENT
        """
        Override this method to do something at every time step
        """
        pass
    
    def spike_call(self, neurons): # TO IMPLEMENT
        """
        Override this method to do something at every time spike. neurons contains
        the list of neurons that just spiked
        """
        pass
    
    def get_value(self, varname):
        return self.group.state_(varname)
    
    def __call__(self):
        self.timestep_call()
        neurons = self.group.get_spikes()
        self.spike_call(neurons)
        
#    def propagate(self, neurons):
#        self.spike_call(neurons)

    def get_values(self): # TO IMPLEMENT
        """
        Override this method to return the criterion values at the end.
        It must return one (or several, as a tuple) additive values, i.e.,
        values corresponding to slices. The method normalize can normalize
        combined values.
        """
        pass
    additive_values = property(get_values)

    def normalize(self, values): # TO IMPLEMENT
        """
        Values contains combined criterion values. It is either a vector of values (as many values
        as neurons), or a tuple with different vector of as many values as neurons.
        """
        pass

class CriterionStruct(object):
    type = None # 'trace', 'spike' or 'both'
    def get_name(self):
        return self.__class__.__name__
    name = property(get_name)




class LpErrorCriterion(Criterion):
    type = 'traces'
    def initialize(self, p=2, varname='v'):
        """
        Called at the beginning of every iteration. The keyword arguments here are
        specified in modelfitting.initialize_criterion().
        """
        self.p = double(p)
        self.varname = varname
        self._error = zeros((self.K, self.N))
    
    def timestep_call(self):
        v = self.get_value(self.varname)
        t = self.step()+1
        if t<self.onset: return # onset
        if t*self.dt >= self.duration: return
        d = self.intdelays
        indices = (t-d>=0)&(t-d<self.total_steps) # neurons with valid delays (stay inside the target trace)
        vtar = self.traces[:,t-d] # target value at this timestep for every neuron
        for i in xrange(self.K):
            self._error[i,indices] += abs(v[indices]-vtar[i,indices])**self.p
    
    def get_cuda_code(self):
        code = {}
        
        #  DECLARATION
        code['%CRITERION_DECLARE%'] = """
    double *error_arr,
    double p,
        """
        
        # INITIALIZATION
        code['%CRITERION_INIT%'] = """
    double error = error_arr[neuron_index];
        """
        
        # TIMESTEP
        code['%CRITERION_TIMESTEP%'] = """
        if (Tdelay<duration-spikedelay-1) {
            error = error + pow(abs(trace_value - %s), %.4f);
        }
        """ % (self.varname, self.p)
        
        # FINALIZATION
        code['%CRITERION_END%'] = """
    error_arr[neuron_index] = error;
        """
        
        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.error_gpu = gpuarray.to_gpu(zeros(self.N, dtype=double))
    
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.error_gpu, self.p]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self._error = self.error_gpu.get()
    
    def get_values(self):
        if self.K == 1: error = self._error.flatten()
        else: error = self._error
        return error # just the integral, for every slice
    
    def normalize(self, error):
        # error is now the combined error on the whole duration (sum on the slices)
        # HACK: 1- because modelfitting MAXIMIZES for now...
#        self._norm = sum(self.traces**self.p, axis=1)**(1./self.p) # norm of every trace
        return 1-(self.dt*error)**(1./self.p)#/self._norm

class LpError(CriterionStruct):
    """
    Structure used by the users to specify a criterion
    """
    def __init__(self, p = 2, varname = 'v'):
        self.type = 'trace'
        self.p = p
        self.varname = varname




class GammaFactorCriterion(Criterion):
    """
    Coincidence counter class.
    
    Counts the number of coincidences between the spikes of the neurons in the network (model spikes),
    and some user-specified data spike trains (target spikes). This number is defined as the number of 
    target spikes such that there is at least one model spike within +- ``delta``, where ``delta``
    is the half-width of the time window.
    
    Initialised as::
    
        cc = CoincidenceCounter(source, data, delta = 4*ms)
    
    with the following arguments:
    
    ``source``
        A :class:`NeuronGroup` object which neurons are being monitored.
    
    ``data``
        The list of spike times. Several spike trains can be passed in the following way.
        Define a single 1D array ``data`` which contains all the target spike times one after the
        other. Now define an array ``spiketimes_offset`` of integers so that neuron ``i`` should 
        be linked to target train: ``data[spiketimes_offset[i]], data[spiketimes_offset[i]+1]``, etc.
        
        It is essential that each spike train with the spiketimes array should begin with a spike at a
        large negative time (e.g. -1*second) and end with a spike that is a long time
        after the duration of the run (e.g. duration+1*second).
    
    ``delta=4*ms``
        The half-width of the time window for the coincidence counting algorithm.
    
    ``spiketimes_offset``
        A 1D array, ``spiketimes_offset[i]`` is the index of the first spike of 
        the target train associated to neuron i.
        
    ``spikedelays``
        A 1D array with spike delays for each neuron. All spikes from the target 
        train associated to neuron i are shifted by ``spikedelays[i]``.
        
    ``coincidence_count_algorithm``
        If set to ``'exclusive'``, the algorithm cannot count more than one
        coincidence for each model spike.
        If set to ``'inclusive'``, the algorithm can count several coincidences
        for a single model spike.
    
    ``onset``
        A scalar value in seconds giving the start of the counting: no
        coincidences are counted before ``onset``.
    
    Has three attributes:
    
    ``coincidences``
        The number of coincidences for each neuron of the :class:`NeuronGroup`.
        ``coincidences[i]`` is the number of coincidences for neuron i.
        
    ``model_length``
        The number of spikes for each neuron. ``model_length[i]`` is the spike
        count for neuron i.
        
    ``target_length``
        The number of spikes in the target spike train associated to each neuron.
    """
    type = 'spikes'
    def initialize(self, delta, coincidence_count_algorithm):
        self.algorithm = coincidence_count_algorithm
        self.delta = int(rint(delta / self.dt))
        
        self.spike_count = zeros(self.N, dtype='int')
        self.coincidences = zeros(self.N, dtype='int')
        self.spiketime_index = self.spikes_offset
        self.last_spike_time = array(rint(self.spikes_inline[self.spiketime_index] / self.dt), dtype=int)
        self.next_spike_time = array(rint(self.spikes_inline[self.spiketime_index + 1] / self.dt), dtype=int)

        # First target spikes (needed for the computation of 
        #   the target train firing rates)
#        self.first_target_spike = zeros(self.N)

        self.last_spike_allowed = ones(self.N, dtype='bool')
        self.next_spike_allowed = ones(self.N, dtype='bool')
        
    def spike_call(self, spiking_neurons):
        dt = self.dt
        t = self.step()*dt
        
        spiking_neurons = array(spiking_neurons)
        if len(spiking_neurons)>0:
            
            if t >= self.onset:
                self.spike_count[spiking_neurons] += 1

            T_spiking = array(rint((t + self.delays[spiking_neurons]) / dt), dtype=int)

            remaining_neurons = spiking_neurons
            remaining_T_spiking = T_spiking
            while True:
                remaining_indices, = (remaining_T_spiking > self.next_spike_time[remaining_neurons]).nonzero()
                if len(remaining_indices):
                    indices = remaining_neurons[remaining_indices]
                    self.spiketime_index[indices] += 1
                    self.last_spike_time[indices] = self.next_spike_time[indices]
                    self.next_spike_time[indices] = array(rint(self.spikes_inline[self.spiketime_index[indices] + 1] / dt), dtype=int)
                    if self.algorithm == 'exclusive':
                        self.last_spike_allowed[indices] = self.next_spike_allowed[indices]
                        self.next_spike_allowed[indices] = True
                    remaining_neurons = remaining_neurons[remaining_indices]
                    remaining_T_spiking = remaining_T_spiking[remaining_indices]
                else:
                    break

            # Updates coincidences count
            near_last_spike = self.last_spike_time[spiking_neurons] + self.delta >= T_spiking
            near_next_spike = self.next_spike_time[spiking_neurons] - self.delta <= T_spiking
            last_spike_allowed = self.last_spike_allowed[spiking_neurons]
            next_spike_allowed = self.next_spike_allowed[spiking_neurons]
            I = (near_last_spike & last_spike_allowed) | (near_next_spike & next_spike_allowed)

            if t >= self.onset:
                self.coincidences[spiking_neurons[I]] += 1

            if self.algorithm == 'exclusive':
                near_both_allowed = (near_last_spike & last_spike_allowed) & (near_next_spike & next_spike_allowed)
                self.last_spike_allowed[spiking_neurons] = last_spike_allowed & -near_last_spike
                self.next_spike_allowed[spiking_neurons] = (next_spike_allowed & -near_next_spike) | near_both_allowed
    
    def get_cuda_code(self):
        code = {}
        
        #  DECLARATION
        code['%CRITERION_DECLARE%'] = """
    int *spikecount,          // Number of spikes produced by each neuron
    int *num_coincidences,    // Count of coincidences for each neuron
        """
        
        if self.algorithm == 'exclusive':
            code['%CRITERION_DECLARE%'] += """ 
    bool *last_spike_allowed_arr,
    bool *next_spike_allowed_arr,
           """
        
        # INITIALIZATION
        code['%CRITERION_INIT%'] = """
    int ncoinc = num_coincidences[neuron_index];
    int nspikes = spikecount[neuron_index];
    int last_spike_time = spiketimes[spiketime_index];
    int next_spike_time = spiketimes[spiketime_index+1];
        """
        
        if self.algorithm == 'exclusive':
            code['%CRITERION_INIT%'] += """
    bool last_spike_allowed = last_spike_allowed_arr[neuron_index];
    bool next_spike_allowed = next_spike_allowed_arr[neuron_index];
            """
        
        # TIMESTEP
        code['%CRITERION_TIMESTEP%'] = """
        const int Tspike = T+spikedelay;
        """
        
        if self.algorithm == 'inclusive':
            code['%CRITERION_TIMESTEP%'] += """ 
    ncoinc += has_spiked && (((last_spike_time+%d)>=Tspike) || ((next_spike_time-%d)<=Tspike));
            """ % (self.delta, self.delta)
        if self.algorithm == 'exclusive':
            code['%CRITERION_TIMESTEP%'] += """
        bool near_last_spike = last_spike_time+%d>=Tspike;
        bool near_next_spike = next_spike_time-%d<=Tspike;
        near_last_spike = near_last_spike && has_spiked;
        near_next_spike = near_next_spike && has_spiked;
        ncoinc += (near_last_spike&&last_spike_allowed) || (near_next_spike&&next_spike_allowed);
        bool near_both_allowed = (near_last_spike&&last_spike_allowed) && (near_next_spike&&next_spike_allowed);
        last_spike_allowed = last_spike_allowed && !near_last_spike;
        next_spike_allowed = (next_spike_allowed && !near_next_spike) || near_both_allowed;
            """ % (self.delta, self.delta)
    
        code['%CRITERION_TIMESTEP%'] += """
        nspikes += has_spiked*(T>=onset);
        if(Tspike>=next_spike_time){
            spiketime_index++;
            last_spike_time = next_spike_time;
            next_spike_time = spiketimes[spiketime_index+1];
        """
    
        if self.algorithm == 'exclusive':
            code['%CRITERION_TIMESTEP%'] += """
        last_spike_allowed = next_spike_allowed;
        next_spike_allowed = true;
            """
    
        code['%CRITERION_TIMESTEP%'] += """
        }
        """
        
        # FINALIZATION
        code['%CRITERION_END%'] = """
    num_coincidences[neuron_index] = ncoinc;
    spikecount[neuron_index] = nspikes;
        """
        
        if self.algorithm == 'exclusive':
            code['%CRITERION_END%'] += """
    last_spike_allowed_arr[neuron_index] = last_spike_allowed;
    next_spike_allowed_arr[neuron_index] = next_spike_allowed;
            """
        
        return code
    
    def initialize_cuda_variables(self):
        """
        Initialize GPU variables to pass to the kernel
        """
        self.spike_count_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
        self.coincidences_gpu = gpuarray.to_gpu(zeros(self.N, dtype=int32))
        
        if self.algorithm == 'exclusive':
            self.last_spike_allowed_arr = gpuarray.to_gpu(zeros(self.N, dtype=bool))
            self.next_spike_allowed_arr = gpuarray.to_gpu(ones(self.N, dtype=bool))
    
    def get_kernel_arguments(self):
        """
        Return a list of objects to pass to the CUDA kernel.
        """
        args = [self.spike_count_gpu, self.coincidences_gpu]
        
        if self.algorithm == 'exclusive':
            args += [self.last_spike_allowed_arr,
                     self.next_spike_allowed_arr]
        return args
    
    def update_gpu_values(self):
        """
        Call gpuarray.get() on final values, so that get_values() returns updated values.
        """
        self.coincidences = self.coincidences_gpu.get()
        self.spike_count = self.spike_count_gpu.get()
    
    def get_values(self):
        return (self.coincidences, self.spike_count)
    
    def normalize(self, values):
        coincidence_count = values[0]
        spike_count = values[1]
        delta = self.delta*self.dt

        gamma = get_gamma_factor(coincidence_count, spike_count, 
                                 self.target_spikes_count, self.target_spikes_rate, 
                                 delta)
        
        return gamma

class GammaFactor(CriterionStruct):
    def __init__(self, delta = 4*ms, coincidence_count_algorithm = 'exclusive'):
        self.type = 'spikes'
        self.delta = delta
        self.coincidence_count_algorithm = coincidence_count_algorithm




class VanRossumCriterion(Criterion):
    type = 'spikes'
    def initialize(self, p=2, varname='v'):
        # TODO
        pass
    
    def timestep_call(self):
        # TODO
        pass
    
    def get_values(self):
        # TODO
        pass
    
    def normalize(self, error):
        # TODO
        pass

class VanRossum(CriterionStruct):
    # TODO
    def __init__(self, tau):
        self.type = 'spikes'
        self.tau = tau

